/************************************************************************************
**    SIMphoto is an aplication based on single image photogrammetry. It is useful
**    for creating and printing a photomap. The software is designed for close
**    range photogrammtery. Outputs are usable in the field of heritage preservation.
**    Copyright (C) 2009 David Cizek
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program. If not, see <http://www.gnu.org/licenses/>.
**
**    How to contact me: cizekdav@gmail.com
**
***********************************************************************************/

#include "clipimagedialog.h"

ClipImageDialog::ClipImageDialog(QImage img, int imageWidth, QWidget *parent)
    : QDialog(parent)
{
    ratio = 1.0 * imageWidth / img.width();

    graphicsView = new QGraphicsView;
    graphicsScene = new QGraphicsScene;
    graphicsView->setScene(graphicsScene);

    Qt::WindowFlags flags;
    flags = Qt::Dialog;
    flags |= Qt::WindowCloseButtonHint;

    QPainterPath path;
    path.moveTo(0, 0);
    path.lineTo(0, 10);
    path.moveTo(0, 0);
    path.lineTo(0, -10);
    path.moveTo(0, 0);
    path.lineTo(10, 0);
    path.moveTo(0, 0);
    path.lineTo(-10, 0);

    printRectangle = new QRect();
    pixmapItem = new QGraphicsPixmapItem(QPixmap::fromImage(img));
    rectItem = new QGraphicsRectItem(0, 0, 0, 0);
    crossItem = new QGraphicsPathItem(path);

    rectItem->setZValue(1);
    crossItem->setZValue(2);

    graphicsScene->setSceneRect(0, 0, img.width(), img.height());
    graphicsScene->addItem(pixmapItem);
    graphicsScene->addItem(rectItem);
    graphicsScene->addItem(crossItem);

    graphicsView->viewport()->installEventFilter(this);
    graphicsView->viewport()->setMouseTracking(true);
    graphicsView->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    graphicsView->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    graphicsView->setFixedSize((int)graphicsScene->width(), (int)graphicsScene->height());
    graphicsView->viewport()->setCursor(Qt::CrossCursor);

    QLabel *label = new QLabel(tr("First click to select the upper left corner, second click to select lower right corner."),this);
    nextButton = new QPushButton(tr("Next >>"), this);
    nextButton->setEnabled(false);

    QSpacerItem *spacer = new QSpacerItem(50, 8, QSizePolicy::Expanding, QSizePolicy::Minimum);

    QGridLayout *layout = new QGridLayout;
    QHBoxLayout *hLayout = new QHBoxLayout;

    hLayout->addItem(spacer);
    hLayout->addWidget(nextButton);

    layout->addWidget(label, 0, 0);
    layout->addWidget(graphicsView, 1, 0);
    layout->addLayout(hLayout, 2, 0);

    clipArea = false;
    firstClick = true;

    setWindowFlags(flags);
    this->setWindowTitle(tr("Printable Area Selection"));
    setLayout(layout);
    setModal(true);
}

bool ClipImageDialog::eventFilter(QObject *obj, QEvent *event)
{
    if(obj == graphicsView->viewport())
    {
        if(event->type() == QEvent::MouseMove && !firstClick)
        {
            QMouseEvent *moveEvent = static_cast<QMouseEvent*>(event);

            QBrush brush(QColor(170, 0, 0, 100), Qt::SolidPattern);
            rectItem->setPen(QPen(QColor(170, 0, 0), 2, Qt::SolidLine,Qt::FlatCap, Qt::MiterJoin));
            rectItem->setBrush(brush);

            if(moveEvent->x() - rectX < 0 || moveEvent->y() - rectY < 0)
                rectItem->setRect(rectX, rectY, 0, 0);
            else
                rectItem->setRect(rectX, rectY, moveEvent->x() - rectX, moveEvent->y() - rectY);

            rectItem->show();

            return true;
        }
        else if(event->type() == QEvent::MouseButtonPress)
        {
            if(firstClick)
            {
                //localization of upper left corner
                QMouseEvent *pressEvent = static_cast<QMouseEvent*>(event);

                rectItem->hide();

                crossItem->setPos(pressEvent->x(), pressEvent->y());
                crossItem->setPen(QPen(QColor(170, 0, 0), 2, Qt::SolidLine,Qt::FlatCap, Qt::MiterJoin));

                rectX = pressEvent->x();
                rectY = pressEvent->y();

                nextButton->setEnabled(false);
                clipArea = false;
                firstClick = false;
            }
            else
            {
                //localization of buttom right corner
                QBrush brush(QColor(255, 201, 14, 100), Qt::SolidPattern);
                rectItem->setPen(QPen(QColor(255, 201, 14), 2, Qt::SolidLine,Qt::FlatCap, Qt::MiterJoin));
                rectItem->setBrush(brush);
                crossItem->setPen(QPen(QColor(255, 201, 14), 2, Qt::SolidLine,Qt::FlatCap, Qt::MiterJoin));

               if(rectItem->rect().width() != 0 || rectItem->rect().height() != 0)
                    nextButton->setEnabled(true);

                clipArea = true;
                firstClick = true;
            }

            return true;
        }
        else
        {
            return false;
        }
    }
    else
    {
         return QDialog::eventFilter(obj, event);
    }
}

//return selected rectangle
QRect* ClipImageDialog::getPrintRectangle()
{
    printRectangle->setX((int)(rectItem->rect().x() * ratio));
    printRectangle->setY((int)(rectItem->rect().y() * ratio));
    printRectangle->setWidth((int)(rectItem->rect().width() * ratio));
    printRectangle->setHeight((int)(rectItem->rect().height() * ratio));

    return printRectangle;
}

QPushButton* ClipImageDialog::getNextButton()
{
    return nextButton;
}

void ClipImageDialog::closeEvent(QCloseEvent *closeEvent)
{
    delete this;
}
