/************************************************************************************
**    SIMphoto is an aplication based on single image photogrammetry. It is useful
**    for creating and printing a photomap. The software is designed for close
**    range photogrammtery. Outputs are usable in the field of heritage preservation.
**    Copyright (C) 2009 David Cizek
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program. If not, see <http://www.gnu.org/licenses/>.
**
**    How to contact me: cizekdav@gmail.com
**
***********************************************************************************/

#include "similartransformdialog.h"

SimilarTransformDialog::SimilarTransformDialog(double pixelSize, QWidget *parent)
    : QDialog(parent)
{
    changeScaleCheckBox = new QCheckBox(tr("Change Pixel Size"), this);
    rotateCheckBox = new QCheckBox(tr("Rotate"), this);
    currentPixelSizeLabel = new QLabel(this);
    newPixelSizeLabel = new QLabel(this);
    newScaleSpinBox = new QDoubleSpinBox(this);
    percentageScaleSpinBox = new QSpinBox(this);
    meterLabel = new QLabel("m", this);

    if(pixelSize == 0)
    {
        relativeScale();
    }
    else
    {
        newPixelSize(pixelSize);
    }

    pointMask = QBitmap("images/points/mask.bmp");

    textPointLabel = new QLabel;
    pointLabel = new QLabel;

    pointNumberSpinBox = new QSpinBox;

    pointNumberSpinBox->setRange(1, 2);
    pointNumberSpinBox->setValue(1);

    valueWasChanged(1);

    transformButton = new QPushButton(tr("Transform"));

    transformButton->setEnabled(false);
    currentPixelSizeLabel->setEnabled(false);
    newPixelSizeLabel->setEnabled(false);
    meterLabel->setEnabled(false);
    newScaleSpinBox->setEnabled(false);
    percentageScaleSpinBox->setEnabled(false);
    textPointLabel->setEnabled(false);
    pointLabel->setEnabled(false);
    pointNumberSpinBox->setEnabled(false);

    QGridLayout *layout = new QGridLayout;

    layout->addWidget(rotateCheckBox, 0, 0);
    layout->addWidget(changeScaleCheckBox, 1, 0);
    layout->addWidget(textPointLabel, 2, 0);
    layout->addWidget(pointNumberSpinBox, 2, 1);
    layout->addWidget(pointLabel, 2, 2);
    layout->addWidget(currentPixelSizeLabel, 3, 0);
    layout->addWidget(transformButton, 5, 0);

    if(pixelSize == 0)
    {
        layout->addWidget(newPixelSizeLabel, 4, 0);
        layout->addWidget(percentageScaleSpinBox, 4, 1);
    }
    else
    {
        layout->addWidget(newPixelSizeLabel, 4, 0);
        layout->addWidget(newScaleSpinBox, 4, 1);
        layout->addWidget(meterLabel, 4, 2);
    }

    Qt::WindowFlags flags;
    flags = Qt::Dialog;
    flags |= Qt::WindowCloseButtonHint;

    setWindowFlags(flags);
    setWindowTitle(tr("Rotate / Change Pixel Size"));
    setLayout(layout);

    connect(pointNumberSpinBox, SIGNAL(valueChanged(int)), this, SLOT(valueWasChanged(int)));
    connect(changeScaleCheckBox, SIGNAL(stateChanged(int)), this, SLOT(scale()));
    connect(rotateCheckBox, SIGNAL(stateChanged(int)), this, SLOT(rotation()));
}

void SimilarTransformDialog::scale()
{
    if(changeScaleCheckBox->isChecked())
    {
        currentPixelSizeLabel->setEnabled(true);
        newPixelSizeLabel->setEnabled(true);
        meterLabel->setEnabled(true);
        newScaleSpinBox->setEnabled(true);
        percentageScaleSpinBox->setEnabled(true);
    }
    else
    {
        currentPixelSizeLabel->setEnabled(false);
        newPixelSizeLabel->setEnabled(false);
        meterLabel->setEnabled(false);
        newScaleSpinBox->setEnabled(false);
        percentageScaleSpinBox->setEnabled(false);
    }

    enableTransButton();
}

void SimilarTransformDialog::rotation()
{
    if(rotateCheckBox->isChecked())
    {
        rotateIsChecked();
        textPointLabel->setEnabled(true);
        pointLabel->setEnabled(true);
        pointNumberSpinBox->setEnabled(true);
    }
    else
    {
        rotateIsUnchecked();
        textPointLabel->setEnabled(false);
        pointLabel->setEnabled(false);
        pointNumberSpinBox->setEnabled(false);
    }

    enableTransButton();
}

void SimilarTransformDialog::enableTransButton()
{
    if(changeScaleCheckBox->isChecked() || rotateCheckBox->isChecked())
        transformButton->setEnabled(true);
    else
        transformButton->setEnabled(false);
}

void SimilarTransformDialog::valueWasChanged(int value)
{
    QString pointFile("images/points/");
    pointFile.append(QString::number(value)).append(".bmp");

    QPixmap *pixmap = new QPixmap(pointFile);

    pixmap->setMask(pointMask);
    pointLabel->setPixmap(*pixmap);

    if(value == 1)
    {
        textPointLabel->setText(tr("Mark the top point of vertical."));
        firstPoint();
    }
    else
    {
        textPointLabel->setText(tr("Mark the bottom point of vertical."));
        secondPoint();
    }

    delete pixmap;
}

void SimilarTransformDialog::relativeScale()
{
    meterLabel->setHidden(true);
    currentPixelSizeLabel->setHidden(true);
    newScaleSpinBox->setHidden(true);
    newPixelSizeLabel->setText(tr("Type a new image size (percentage [%])"));

    percentageScaleSpinBox->setRange(10, 100);
    percentageScaleSpinBox->setValue(100);
}

void SimilarTransformDialog::newPixelSize(double size)
{
    percentageScaleSpinBox->setHidden(true);

    QString sizeString(tr("Curent pixel size: 1 pixel = "));
    sizeString.append(QString::number(size, 'f', 4)).append(" m");

    currentPixelSizeLabel->setText(sizeString);
    newPixelSizeLabel->setText(tr("Type a new pixel size: 1 pixel ="));

    newScaleSpinBox->setDecimals(4);
    newScaleSpinBox->setRange(0.0001, 100.0);
    newScaleSpinBox->setValue(size);
}

QSpinBox* SimilarTransformDialog::getPointSpinBox()
{
    return pointNumberSpinBox;
}

QSpinBox* SimilarTransformDialog::getPercentageScaleSpinBox()
{
    return percentageScaleSpinBox;
}

QDoubleSpinBox* SimilarTransformDialog::getNewScaleSpinBox()
{
    return newScaleSpinBox;
}

QCheckBox* SimilarTransformDialog::getChangeScaleCheckBox()
{
    return changeScaleCheckBox;
}


QCheckBox* SimilarTransformDialog::getRotateCheckBox()
{
    return rotateCheckBox;
}

QPushButton* SimilarTransformDialog::getTransformButton()
{
    return transformButton;
}
