/************************************************************************************
**    SIMphoto is an aplication based on single image photogrammetry. It is useful
**    for creating and printing a photomap. The software is designed for close
**    range photogrammtery. Outputs are usable in the field of heritage preservation.
**    Copyright (C) 2009 David Cizek
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program. If not, see <http://www.gnu.org/licenses/>.
**
**    How to contact me: cizekdav@gmail.com
**
***********************************************************************************/

#include "simphoto.h"

SIMphoto::SIMphoto(QWidget *parent)
    : QMainWindow(parent), image(NULL), overviewLabel(NULL), markingTool(NULL), cursorColorDialog(NULL)
{
    QIcon windowIcon("images/windowIcon.png");

    showMaximized();
    setWindowIcon(windowIcon);

    createActions();
    createMenus();
    createToolBars();
    createCentralWidget();
    createZoom();

    blackCursorColor = true;

    //centralWidget()->installEventFilter(this);
    imageScrollArea->installEventFilter(this);
    statusBar()->showMessage("SIMphoto version 12.2010 Copyright (C) 2010 David Cizek; This program comes with ABSOLUTELY NO WARRANTY; This is free software, and you are welcome to redistribute it under certain conditions; see licence.txt for details.");
}

void SIMphoto::createActions()
{
    openAct = new QAction(QIcon("images/openIcon.bmp"), tr("Open Image"), this);
    openAct->setStatusTip(tr("Open Image"));
    connect(openAct, SIGNAL(triggered()), this, SLOT(openImage()));

    saveAct = new QAction(QIcon("images/saveIcon.bmp"), tr("&Save Image As"), this);
    saveAct->setStatusTip(tr("Save Image As"));
    saveAct->setEnabled(false);
    connect(saveAct, SIGNAL(triggered()), this, SLOT(saveImage()));

    printAct = new QAction(QIcon("images/printIcon.bmp"), tr("&Print Image To PDF"), this);
    printAct->setStatusTip(tr("Print Image To PDF"));
    printAct->setEnabled(false);
    connect(printAct, SIGNAL(triggered()), this, SLOT(printImage()));

    exitAct = new QAction(tr("&Exit"), this);
    exitAct->setStatusTip(tr("Exit"));
    connect(exitAct, SIGNAL(triggered()), this, SLOT(exitApl()));

    undoAct = new QAction(QIcon("images/undoIcon.bmp"), tr("&Undo"), this);
    undoAct->setStatusTip(tr("Undo"));
    connect(undoAct, SIGNAL(triggered()), this, SLOT(undo()));

    controlPointsMethodAct = new QAction(QIcon("images/controlIcon.bmp"), tr("&Control Points Method"), this);
    controlPointsMethodAct->setStatusTip(tr("Control Points Method"));
    connect(controlPointsMethodAct, SIGNAL(triggered()), this, SLOT(doControlPointsMethodAction()));

    networkMethodAct = new QAction(QIcon("images/networkIcon.bmp"), tr("&Network Method"), this);
    networkMethodAct->setStatusTip(tr("Network Method"));
    connect(networkMethodAct, SIGNAL(triggered()), this, SLOT(doNetworkMethodAction()));

    distMethodAct = new QAction(QIcon("images/distanceIcon.bmp"), tr("&Separated Distance/s Method"), this);
    distMethodAct->setStatusTip(tr("Separated Distance/s Method"));
    connect(distMethodAct, SIGNAL(triggered()), this, SLOT(doDistMethodAction()));

    similarTransAct = new QAction(QIcon("images/similarIcon.bmp"), tr("&Rotate / Change Pixel Size"), this);
    similarTransAct->setStatusTip(tr("Rotate / Change Pixel Size"));
    connect(similarTransAct, SIGNAL(triggered()), this, SLOT(doSimilarTransAction()));

    distorsionRedAct = new QAction(QIcon("images/distorsionIcon.bmp"), tr("&Distorsion Reduction"), this);
    distorsionRedAct->setStatusTip(tr("Distorsion Reduction"));
    connect(distorsionRedAct, SIGNAL(triggered()), this, SLOT(doDistorsionRedAction()));

    loadTableAct = new QAction(QIcon("images/tableIcon.bmp"), tr("&Load an Existing Table"), this);
    loadTableAct->setStatusTip(tr("Load an Existing Table"));
    connect(loadTableAct, SIGNAL(triggered()), this, SLOT(doLoadTableAction()));

    measureAct = new QAction(QIcon("images/measureIcon.bmp"), tr("&Measure Distances"), this);
    measureAct->setStatusTip(tr("Measure Distances"));
    connect(measureAct, SIGNAL(triggered()), this, SLOT(doMeasureAction()));

    scalebarAct = new QAction(QIcon("images/scaleIcon.bmp"), tr("&Add Scale Bar"), this);
    scalebarAct->setStatusTip(tr("Add Scale Bar"));
    connect(scalebarAct, SIGNAL(triggered()), this, SLOT(doScalebarAction()));

    overviewAct = new QAction(QIcon("images/overviewIcon.bmp"), tr("&Overview"), this);
    overviewAct->setStatusTip(tr("Overview"));
    connect(overviewAct, SIGNAL(triggered()), this, SLOT(doOverviewAction()));

    cursorColorAct = new QAction(QIcon("images/cursorIcon.bmp"), tr("&Cursor Color"), this);
    cursorColorAct->setStatusTip(tr("Cursor Color"));
    connect(cursorColorAct, SIGNAL(triggered()), this, SLOT(doCursorColorAction()));

    zoom3xAct = new QAction(tr("&3x Zoom"), this);
    zoom3xAct->setStatusTip(tr("3x Zoom"));
    connect(zoom3xAct, SIGNAL(triggered()), this, SLOT(doZoom3xAction()));

    zoom5xAct = new QAction(tr("&5x Zoom"), this);
    zoom5xAct->setStatusTip(tr("5x Zoom"));
    connect(zoom5xAct, SIGNAL(triggered()), this, SLOT(doZoom5xAction()));

    noZoomAct = new QAction(tr("&No Zoom"), this);
    noZoomAct->setStatusTip(tr("No Zoom"));
    connect(noZoomAct, SIGNAL(triggered()), this, SLOT(doNoZoomAction()));

    aboutAct = new QAction(QIcon("images/windowIcon.png"), tr("&About SIMphoto"), this);
    aboutAct->setStatusTip(tr("About SIMphoto"));
    connect(aboutAct, SIGNAL(triggered()), this, SLOT(doAboutAction()));
}

void SIMphoto::createMenus()
{
    imageMenu = menuBar()->addMenu(tr("&Image"));
    imageMenu->addAction(openAct);
    imageMenu->addAction(saveAct);
    imageMenu->addSeparator();
    imageMenu->addAction(printAct);
    imageMenu->addSeparator();
    imageMenu->addAction(exitAct);

    editMenu = menuBar()->addMenu(tr("&Edit"));
    editMenu->addAction(undoAct);
    editMenu->setEnabled(false);

    photomapMenu = menuBar()->addMenu(tr("&Photomap"));
    photomapMenu->addAction(controlPointsMethodAct);
    photomapMenu->addAction(networkMethodAct);
    photomapMenu->addAction(distMethodAct);
    photomapMenu->setEnabled(false);

    functionsMenu = menuBar()->addMenu(tr("&Functions"));
    functionsMenu->addAction(similarTransAct);
    functionsMenu->addAction(distorsionRedAct);
    functionsMenu->addAction(loadTableAct);
    functionsMenu->addSeparator();
    functionsMenu->addAction(measureAct);
    functionsMenu->addAction(scalebarAct);
    functionsMenu->setEnabled(false);

    viewMenu = menuBar()->addMenu(tr("&View"));
    viewMenu->addAction(overviewAct);
    viewMenu->addAction(cursorColorAct);
    viewMenu->setEnabled(false);

    zoomMenu = viewMenu->addMenu(tr("&Zoom"));
    zoomMenu->addAction(zoom3xAct);
    zoomMenu->addAction(zoom5xAct);
    zoomMenu->addAction(noZoomAct);

    helpMenu = menuBar()->addMenu(tr("&Help"));
    helpMenu->addAction(aboutAct);
}

void SIMphoto::createToolBars()
 {
     imageToolBar = addToolBar(tr("Image"));
     imageToolBar->addAction(openAct);
     imageToolBar->addAction(saveAct);
     imageToolBar->addAction(printAct);

     photomapToolBar = addToolBar(tr("Photomap"));
     photomapToolBar->addAction(controlPointsMethodAct);
     photomapToolBar->addAction(networkMethodAct);
     photomapToolBar->addAction(distMethodAct);
     photomapToolBar->setEnabled(false);

     functionsToolBar = addToolBar(tr("Function"));
     functionsToolBar->addAction(similarTransAct);
     functionsToolBar->addAction(distorsionRedAct);
     functionsToolBar->addAction(loadTableAct);
     functionsToolBar->addSeparator();
     functionsToolBar->addAction(measureAct);
     functionsToolBar->addAction(scalebarAct);
     functionsToolBar->setEnabled(false);

     viewToolBar = addToolBar(tr("View"));
     viewToolBar->addAction(overviewAct);
     viewToolBar->addAction(cursorColorAct);
     viewToolBar->setEnabled(false);
}

void SIMphoto::createCentralWidget()
{
    imageLabel = new QLabel;
    imageLabel->installEventFilter(this);
    imageLabel->setMouseTracking(true);

    imageScrollArea = new QScrollArea;
    imageScrollArea->setBackgroundRole(QPalette::Dark);
    imageScrollArea->setWidget(imageLabel);

    setCentralWidget(imageScrollArea);

    //When a scrollbar changes its value, overview moves the frame to the right position.
    connect(imageScrollArea->horizontalScrollBar(), SIGNAL(valueChanged(int)), this, SLOT(setFramePos()));
    connect(imageScrollArea->verticalScrollBar(), SIGNAL(valueChanged(int)), this, SLOT(setFramePos()));
}

void SIMphoto::createZoom()
{
    zoomImage = new QImage(41, 41, QImage::Format_ARGB32);

    zoomLabel = new QLabel(imageLabel);
    zoomLabel->installEventFilter(this);
    zoomLabel->setPixmap(QPixmap::fromImage(*zoomImage));
    zoomLabel->hide();

    //ZoomInt is equal to zoom, so if its value is 1, no zoom will be created.
    zoomInt = 3;
    zoomIsActive = false;
}

void SIMphoto::openImage()
{
    //getting actual directory
    QString path;
    getInitPath(path);

    QString fileName = QFileDialog::getOpenFileName(this,
                                     tr("Open File"), path,
                                     "JPG (*.JPG *.JPEG);; TIF (*.TIF *.TIFF)");
    if (!fileName.isEmpty())
    {
        //If the image was modified...
        if(memory.getNumberOfChanges() != 0)
        {
            bool cancel = false;
            cancel = maybeSave();

            //If user pressed cancel button, cancel variable is true.
            if(cancel)
                return;
        }

        if(image)
            image->~QImage();

        image = new QImage(fileName);
        if(image->isNull())
        {
            QMessageBox::information(this, tr("SIMphoto"),
                                     tr("Cannot load %1.").arg(fileName));
            return;
        }

        initImage(fileName);
        saveInitPath(fileName);

        if(overviewLabel)
        {
            //setting overview parameters
            overviewLabel->setOverview(image, imageScrollArea->size(), imageScrollArea->widget()->pos());
        }
        else
        {
            createOverview();
            overviewLabel->setOverview(image, imageScrollArea->size(), imageScrollArea->widget()->pos());
        }
    }
}

void SIMphoto::getInitPath(QString& initPath)
{
    QFile file("initPath.txt");

    //reading path from txt file
    if(file.open(QIODevice::ReadOnly))
    {
        QTextStream stream(&file);
        initPath = stream.readLine();
    }
}

//When the image was modified and user opens a new image, function "maybeSave()" displays warning message box.
bool SIMphoto::maybeSave()
{
    //Function returns true in case user will press cancel button.
    bool test = false;

    QMessageBox saveChanges(this);
    saveChanges.setText(tr("The image has been modified!"));

    QAbstractButton *saveButton = saveChanges.addButton(tr("Save Image As"), QMessageBox::ActionRole);
    QAbstractButton *openButton = saveChanges.addButton(tr("Open Image"), QMessageBox::ActionRole);
    QAbstractButton *cancelButton = saveChanges.addButton("Cancel", QMessageBox::ActionRole);

    saveChanges.exec();

    if(saveChanges.clickedButton() == saveButton)
    {
        saveImage();
        test = false;
    }
    if(saveChanges.clickedButton() == openButton)
    {
        saveChanges.close();
        test = false;
    }
    if(saveChanges.clickedButton() == cancelButton)
        test = true;

    return test;
}

void SIMphoto::initImage(QString name)
{
    imageLabel->setPixmap(QPixmap::fromImage(*image));
    imageLabel->adjustSize();

    editMenu->setEnabled(true);
    undoAct->setEnabled(false);
    photomapMenu->setEnabled(true);
    functionsMenu->setEnabled(true);
    viewMenu->setEnabled(true);
    photomapToolBar->setEnabled(true);
    functionsToolBar->setEnabled(true);
    viewToolBar->setEnabled(true);

    saveAct->setEnabled(true);
    printAct->setEnabled(true);

    setWindowTitle("SIMphoto - " + name.right(name.size() - name.lastIndexOf("/") - 1));

    //reading information about pixel size and corners coordinates from txt file...
    name.replace(".","_");
    name.push_back(".txt");

    QString line;
    QFile file(name);

    QRegExp sep("\\s+");
    int i = 1;

    if(file.open(QIODevice::ReadOnly))
    {
        QTextStream in(&file);
        while(!in.atEnd())
        {
            line = in.readLine();

            if(i == 2)
                memory.setPixelSize(line.toDouble());

            if(i == 5)
            {
                cornersCoords(0, 0) = line.section(sep, 0, 0).toDouble();
                cornersCoords(0, 1) = line.section(sep, 1, 1).toDouble();
            }

            if(i == 8)
            {
                cornersCoords(1, 0) = line.section(sep, 0, 0).toDouble();
                cornersCoords(1, 1) = line.section(sep, 1, 1).toDouble();

                memory.setCornersCrd(true);

                measureAct->setEnabled(true);
                scalebarAct->setEnabled(true);

                CoeffForObjCrd();

                break;
            }

            i++;
        }

        if(i != 8 || (memory.getPixelSize() == 0))
        {
            measureAct->setEnabled(false);
            scalebarAct->setEnabled(false);
            printAct->setEnabled(false);

            cornersCoords.setZero();
            memory.setPixelSize(0.0);
            memory.setCornersCrd(false);
            statusBar()->clearMessage();
        }

        if(cornersCoords.sum() == 0)
        {
            memory.setCornersCrd(false);
            statusBar()->clearMessage();
        }
    }
    else
    {
        measureAct->setEnabled(false);
        scalebarAct->setEnabled(false);
        printAct->setEnabled(false);

        cornersCoords.setZero();
        memory.setPixelSize(0.0);
        memory.setCornersCrd(false);
        statusBar()->clearMessage();
    }

    memory.setNumberOfChanges(0);
    memory.setObjCoordsZero();

    imageScrollArea->horizontalScrollBar()->setSliderPosition(0);
    imageScrollArea->verticalScrollBar()->setSliderPosition(0);
}

//saving actual directory (path) to txt file
void SIMphoto::saveInitPath(QString initPath)
{
    QString dir = initPath;
    dir.resize(dir.lastIndexOf("/", -1, Qt::CaseInsensitive) + 1);

    QFile file("initPath.txt");
    if(file.open(QFile::WriteOnly | QFile::Truncate))
        {
            QTextStream out(&file);
            out << dir;
        }
}

void SIMphoto::createOverview()
{
    overviewDock = new QDockWidget(tr("Overview"), this);
    overviewDock->setAllowedAreas(Qt::LeftDockWidgetArea | Qt::RightDockWidgetArea);
    overviewDock->setFeatures(QDockWidget::DockWidgetMovable | QDockWidget::DockWidgetFloatable);

    addDockWidget(Qt::LeftDockWidgetArea, overviewDock);

    overviewLabel = new Overview();
    overviewLabel->setAlignment(Qt::AlignTop);
    overviewLabel->installEventFilter(this);

    overviewDock->setWidget(overviewLabel);

    overviewLabel->setOverview(image, imageScrollArea->size(), imageScrollArea->widget()->pos());

    //If the overview is created during marking, all point marks and lines must be drawn in it.
    if(markingTool)
    {
        if(markingTool->marking())
            drawOverview();
    }

    //the same for scale bar action
    if(memory.getActionName() == "scalebarAction")
    {
        if(scalebarDialog->getFinishButton()->isEnabled())
            drawOverview();
    }
}

void SIMphoto::setFramePos()
{
    if(overviewLabel != 0)
        overviewLabel->moveFrame(image->size(), imageScrollArea->size(), imageScrollArea->widget()->pos());
}

void SIMphoto::saveImage()
{
    QString path;
    getInitPath(path);

    QFileDialog saveImageDialog;
    QString fileName = saveImageDialog.getSaveFileName(this, tr("Save Image"),
                            path,
                            "JPG (*.jpg);; TIF (*.tif)");

    if(!fileName.isEmpty())
    {
        QFile file(fileName);

        image->save(&file);
        statusBar()->showMessage(tr("Image Saved"), 4000);

        memory.setNumberOfChanges(0);

        saveInitPath(fileName);

        //saving pixel size and corners coordinates to txt file...
        fileName.replace(".","_");
        fileName.push_back(".txt");

        QFile data(fileName);
        if(data.open(QFile::WriteOnly | QFile::Truncate))
        {
            QTextStream output(&data);
            output << tr("Pixel Size [m]: ") << "\r\n" << memory.getPixelSize() << "\r\n" << "\r\n"
                   << tr("Coordinates Of the Top Left Corner X Y [m]:") << "\r\n" << cornersCoords(0, 0) << "\t" << cornersCoords(0, 1) << "\r\n" << "\r\n"
                   << tr("Coordinates Of the Bottom Right Corner X Y [m]:") << "\r\n" << cornersCoords(1, 0) << "\t" << cornersCoords(1, 1);
        }
    }
}

//The function shows dialog for selecting print area
void SIMphoto::printImage()
{
    clipImageDialog = new ClipImageDialog(image->scaled(500, 500, Qt::KeepAspectRatio, Qt::FastTransformation), image->width(), this);
    clipImageDialog->show();

    connect(clipImageDialog->getNextButton(), SIGNAL(clicked()), this, SLOT(createMapLayoutDialog()));
}

void SIMphoto::createMapLayoutDialog()
{
    QString path;
    getInitPath(path);

    mapLayoutDialog = new MapLayoutDialog(image, clipImageDialog->getPrintRectangle(), memory.getPixelSize(), path, this);

    delete clipImageDialog;
    mapLayoutDialog->show();

    connect(mapLayoutDialog->getBackButton(), SIGNAL(clicked()), this, SLOT(backToClipImage()));
}

void SIMphoto::backToClipImage()
{
    delete mapLayoutDialog;
    printImage();
}

void SIMphoto::exitApl()
{
    close();
}

void SIMphoto::closeEvent(QCloseEvent *closeEvent)
{
    //if image was modified...
    if(memory.getNumberOfChanges() != 0)
    {
        QMessageBox saveChanges(this);
        saveChanges.setText(tr("The image has been modified!"));

        QAbstractButton *saveButton = saveChanges.addButton(tr("Save Image As"), QMessageBox::ActionRole);
        QAbstractButton *quitButton = saveChanges.addButton(tr("Discard Image"), QMessageBox::ActionRole);
        QAbstractButton *cancelButton = saveChanges.addButton("Cancel", QMessageBox::ActionRole);

        saveChanges.exec();

        if(saveChanges.clickedButton() == saveButton)
        {
            saveImage();
        }
        if(saveChanges.clickedButton() == quitButton)
        {
            qApp->quit();
        }
        if(saveChanges.clickedButton() == cancelButton)
        {
            closeEvent->ignore();
        }
    }
    else
    {
        QMessageBox quitAplication(this);
        quitAplication.setText(tr("Exit the Aplication?"));

        QAbstractButton *yesButton = quitAplication.addButton(tr("Yes"), QMessageBox::ActionRole);
        QAbstractButton *noButton = quitAplication.addButton(tr("No"), QMessageBox::ActionRole);

        quitAplication.exec();

        if(quitAplication.clickedButton() == yesButton)
        {
            return;
        }
        if(quitAplication.clickedButton() == noButton)
        {
            closeEvent->ignore();
        }
    }
}

void SIMphoto::undo()
{
    //showing previous image
    image->~QImage();
    image = new QImage(*memory.getPreviousImage());

    imageLabel->setPixmap(QPixmap::fromImage(*image));
    imageLabel->adjustSize();

    undoAct->setEnabled(false);

    //setting previous parameters
    cornersCoords = previousCornersCoords;
    memory.setObjectCoordsKey(memory.getPreviousObjectCoordsKey());
    memory.setCornersCrd(memory.getPreviousCornersCrd());
    memory.setPixelSize(memory.getPreviousPixelSize());
    memory.reduceNumberOfChanges();

    if(memory.getPixelSize() == 0)
    {
        measureAct->setEnabled(false);
        scalebarAct->setEnabled(false);
        printAct->setEnabled(false);
    }

    if(overviewLabel)
        overviewLabel->setOverview(image, imageScrollArea->size(), imageScrollArea->widget()->pos());
}

void SIMphoto::doControlPointsMethodAction()
{
    disableOtherActions();
    memory.setActionName("controlPointsMethod");
    createMarkingTool(9);
}

void SIMphoto::createMarkingTool(int pointsCount)
{
    markingTool = new MarkTool(pointsCount);

    if(memory.getActionName() == "oneDistMethod")
        markingTool->getOpenButton()->setText(tr("Type Distance"));

    if(memory.getActionName() == "twoDistMethod")
        markingTool->getOpenButton()->setText(tr("Type Distances"));

    //creating dock for marking tool
    markDock = new QDockWidget(tr("Marking"), this);
    markDock->setAllowedAreas(Qt::LeftDockWidgetArea | Qt::RightDockWidgetArea);
    markDock->setFeatures(QDockWidget::DockWidgetMovable | QDockWidget::DockWidgetFloatable);
    markDock->setWidget(markingTool);

    addDockWidget(Qt::LeftDockWidgetArea, markDock);

    connect(markingTool->getExitButton(), SIGNAL(clicked()), this, SLOT(closeMarkingTool()));
    connect(markingTool, SIGNAL(readyToMarkingSignal()), this, SLOT(startMarking()));
    connect(markingTool, SIGNAL(pointAddedBySpinBox()), this, SLOT(showAddedPoint()));
    connect(markingTool, SIGNAL(pointWasRemoved()), this, SLOT(erasePointFromOverview()));
    connect(markingTool->getOpenButton(), SIGNAL(clicked()), this, SLOT(getMethod()));
}

//Start marking puts marks into the image
void SIMphoto::startMarking()
{
    setCrossCursor();

    markingTool->addPoint();
    movePoint(- imageScrollArea->widget()->x(), - imageScrollArea->widget()->y());

    //Number and position of marks depends on actual action
    if(memory.getActionName() == "networkMethod")
    {
        Matrix<int, 3, 2> coords;
        coords << 200, 0,
                  200, 200,
                    0, 200;

        for(int i = 0; i < 3; i++)
        {
            markingTool->getSpinBox()->setValue(i + 2);
            movePoint(- imageScrollArea->widget()->x() + coords(i, 0), - imageScrollArea->widget()->y() + coords(i, 1));
        }
    }

    if(memory.getActionName() == "oneDistMethod" || memory.getActionName() == "twoDistMethod" ||
       memory.getActionName() == "similarTransAction" || memory.getActionName() == "measureAction")
    {
        markingTool->getSpinBox()->setValue(2);
        movePoint(- imageScrollArea->widget()->x(), - imageScrollArea->widget()->y() + 200);
    }

    if(memory.getActionName() == "twoDistMethod")
    {
        markingTool->getSpinBox()->setValue(3);
        movePoint(- imageScrollArea->widget()->x() + 200, - imageScrollArea->widget()->y() + 200);
    }

    markingTool->getSpinBox()->setValue(1);
}

//If a point is added to the list of points. it is located to the up left corner of the scroll area.
void SIMphoto::showAddedPoint()
{
    movePoint(- imageScrollArea->widget()->x(), - imageScrollArea->widget()->y());
}

//moves point to the specified position in raster
void SIMphoto::movePoint(int moveX, int moveY)
{
    markingTool->getPoint()->setParent(imageLabel);
    markingTool->getPoint()->move(moveX, moveY);
    markingTool->getPoint()->show();

    //drawing marks into overview
    if(overviewLabel)
        drawOverview();

    //drawing lines between marks in image... It depends on actual method.
    if(memory.getActionName() == "measureAction" && markingTool->getPointsCount() == 2)
    {
        drawDistLines();
        measureTool->computeDistance(markingTool->getListOfPoints().at(0)->x() + 11, markingTool->getListOfPoints().at(0)->y() + 43,
                                     markingTool->getListOfPoints().at(1)->x() + 11, markingTool->getListOfPoints().at(1)->y() + 43,
                                     memory.getPixelSize());
    }

    if(memory.getActionName() == "similarTransAction" && markingTool->getPointsCount() == 2)
        drawDistLines();

    if(memory.getActionName() == "networkMethod" && markingTool->getPointsCount() == 4)
        drawNetworkPath();

    if(memory.getActionName() == "oneDistMethod" && markingTool->getPointsCount() == 2)
        drawDistLines();

    if(memory.getActionName() == "twoDistMethod" && markingTool->getPointsCount() == 3)
        drawDistLines();
}

void SIMphoto::erasePointFromOverview()
{
    if(overviewLabel)
        drawOverview();
}

void SIMphoto::closeMarkingTool()
{
    if(zoomIsActive)
        zoomLabel->hide();

    markingTool->close();
    delete markingTool;
    delete markDock;

    markingTool = 0;
    markDock = 0;

    imageScrollArea->widget()->setCursor(Qt::ArrowCursor);

    if(overviewLabel)
        overviewLabel->setPixmap(QPixmap::fromImage(image->scaled(400, 400, Qt::KeepAspectRatio, Qt::FastTransformation)));

    imageLabel->setPixmap(QPixmap::fromImage(*image));
    imageLabel->adjustSize();
    enableOtherActions();
}

void SIMphoto::getMethod()
{
    if(memory.getActionName() == "oneDistMethod" || memory.getActionName() == "twoDistMethod")
    {
        createDistInputDialog();
    }
    else
    {
        createTableOfPoints();
    }
}

void SIMphoto::doNetworkMethodAction()
{
    disableOtherActions();
    createNetworkMethodDialog();
}

void SIMphoto::createNetworkMethodDialog()
{
    networkMethodDialog = new NetworkMethodDialog(memory.getNetworkDistances(), memory.getDiagonal(), this);
    networkMethodDialog->show();

    connect(networkMethodDialog, SIGNAL(rejected()), this, SLOT(closeNetworkMethodDialog()));
    connect(networkMethodDialog->getOkButton(), SIGNAL(clicked()), this, SLOT(coordsFromNetwork()));
}

void SIMphoto::closeNetworkMethodDialog()
{
    Matrix<double, 6, 1> distances;

    for(int i = 0; i < 6; i++)
    {
        distances(i) = networkMethodDialog->getListOfDistances().at(i)->value();
        delete networkMethodDialog->getListOfDistances().at(i);
    }

    memory.setNetworkDistances(distances);
    memory.setDiagonal(networkMethodDialog->getDiagonal());

    delete networkMethodDialog;
    enableOtherActions();
}

void SIMphoto::coordsFromNetwork()
{
    int selectedDiagonal = networkMethodDialog->getDiagonal();
    closeNetworkMethodDialog();

    bool nonzeroDistance = true;
    bool triangleCondition = true;
    checkNetworkValidity(nonzeroDistance, triangleCondition, memory.getNetworkDistances(), selectedDiagonal);

    if(!nonzeroDistance)
    {
        doNetworkMethodAction();

        QMessageBox::information(this, tr("Warning"),
                                          tr("Distances cannot be zero!"));
            return;
    }
    if(!triangleCondition)
    {
        doNetworkMethodAction();

        QMessageBox::information(this, tr("Warning"),
                                          tr("Triangle condition is not satisfied!"));
            return;
    }

    computeCoordsFromNetwork(memory.getNetworkDistances(), selectedDiagonal);
}

//chacking of the input validity (triangle condition & nonzero condition)
void SIMphoto::checkNetworkValidity(bool& nonzeroDistance, bool& triangleCondition, Matrix<double, 6, 1> d, int diag)
{
    if(d(0) < 1e-3 || d(1) < 1e-3 || d(2) < 1e-3 || d(3) < 1e-3)
        nonzeroDistance = false;

    if(diag == 0 && (d(4) < 1e-3 || d(5) < 1e-3))
        nonzeroDistance = false;

    if(diag == 1 && d(4) < 1e-3)
        nonzeroDistance = false;

    if(diag == 2 && d(5) < 1e-3)
        nonzeroDistance = false;

    if(diag == 0)
    {
        if(d(0) + d(1) <= d(4) || d(1) + d(4) <= d(0) || d(0) + d(4) <= d(1))
            triangleCondition = false;

        if(d(2) + d(3) <= d(4) || d(3) + d(4) <= d(2) || d(2) + d(4) <= d(3))
            triangleCondition = false;

        if(d(0) + d(3) <= d(5) || d(3) + d(5) <= d(0) || d(0) + d(5) <= d(3))
            triangleCondition = false;

        if(d(1) + d(2) <= d(5) || d(2) + d(5) <= d(1) || d(1) + d(5) <= d(2))
            triangleCondition = false;
    }
    if(diag == 1)
    {
        if(d(0) + d(1) <= d(4) || d(1) + d(4) <= d(0) || d(0) + d(4) <= d(1))
            triangleCondition = false;

        if(d(2) + d(3) <= d(4) || d(3) + d(4) <= d(2) || d(2) + d(4) <= d(3))
            triangleCondition = false;
    }
    if(diag == 2)
    {
        if(d(0) + d(3) <= d(5) || d(3) + d(5) <= d(0) || d(0) + d(5) <= d(3))
            triangleCondition = false;

        if(d(1) + d(2) <= d(5) || d(2) + d(5) <= d(1) || d(1) + d(5) <= d(2))
            triangleCondition = false;
    }
}

void SIMphoto::computeCoordsFromNetwork(Matrix<double, 6, 1> d, int diag)
{
    double delta1, delta2, deviation;

    if(diag == 1)
    {
        delta1 = acos((d(0) * d(0) + d(4) * d(4) - d(1) * d(1)) / (2 * d(0) * d(4)));
        delta2 = acos((d(3) * d(3) + d(4) * d(4) - d(2) * d(2)) / (2 * d(3) * d(4)));

        d(5) = sqrt(d(0) * d(0) + d(3) * d(3) - 2 * d(0) * d(3) * cos(delta1 + delta2));
    }
    if(diag == 2)
    {
        delta1 = acos((d(0) * d(0) + d(5) * d(5) - d(3) * d(3)) / (2 * d(0) * d(5)));
        delta2 = acos((d(1) * d(1) + d(5) * d(5) - d(2) * d(2)) / (2 * d(1) * d(5)));

        d(4) = sqrt(d(0) * d(0) + d(1) * d(1) - 2 * d(0) * d(1) * cos(delta1 + delta2));
    }

    Matrix<double, 4, 2> coordsFromDist5;
    Matrix<double, 4, 2> coordsFromDist6;

    bool angleIsAcceptable = true;

    //computing two different coordinates (system1 from diagonal1 (distance 6), system2 from diagonal2 (distance 5))
    coordsFromDist6 = distanceIntersection(d(5), d(3), d(0), d(2), d(1), true, angleIsAcceptable);
    coordsFromDist5 = distanceIntersection(d(4), d(0), d(1), d(3), d(2), false, angleIsAcceptable);

    if(!angleIsAcceptable)
    {
        doNetworkMethodAction();

        QMessageBox::information(this, tr("Warning"),
                     tr("Intersection angle is not acceptable! It's value has to be between 20 and 160 degrees."));
            return;
    }

    //computing maximal position difference of two systems
    deviation = maxPositionDeviation(coordsFromDist6, coordsFromDist5);

    //Y axes will be put into distance 4
    coordsFromDist6 = putY_axesToDist4(coordsFromDist6, d(3));

    //If both diagonal were measured...
    if(diag == 0)
    {
        showDeviationMessage(deviation, coordsFromDist6);
    }
    else
    {
        disableOtherActions();
        memory.setActionName("networkMethod");
        memory.setNetworkCoordinates(coordsFromDist6);
        createMarkingTool(4);
    }
}

//computing coordinates from measured distances
Matrix<double, 4, 2> SIMphoto::distanceIntersection(double X_Axes, double dA_AboveX, double dB_AboveX,
                                                     double dA_BelowX,  double dB_BelowX, bool isDist6,
                                                     bool& angleIsAcceptable)
{
    Matrix<double, 4, 2> coordinates;
    double omega1, omega2, intersectAngle;

    if(isDist6)
    {
        coordinates(1, 0) = X_Axes;
        coordinates(1, 1) = 0;
        coordinates(3, 0) = 0;
        coordinates(3, 1) = 0;
    }
    else
    {
        coordinates(0, 0) = 0;
        coordinates(0, 1) = 0;
        coordinates(2, 0) = X_Axes;
        coordinates(2, 1) = 0;
    }

    omega1 = acos((dA_AboveX * dA_AboveX + X_Axes * X_Axes - dB_AboveX * dB_AboveX) / (2 * dA_AboveX * X_Axes));
    omega2 = acos((dA_BelowX * dA_BelowX + X_Axes * X_Axes - dB_BelowX * dB_BelowX) / (2 * dA_BelowX * X_Axes));

    intersectAngle = asin(X_Axes * sin(omega1) / dB_AboveX) * 180 / M_PI;

    if (intersectAngle < 20 || intersectAngle > 160)
        angleIsAcceptable = false;

    intersectAngle = asin(X_Axes * sin(omega2) / dB_BelowX) * 180 / M_PI;

    if (intersectAngle < 20 || intersectAngle > 160)
        angleIsAcceptable = false;

    if(isDist6)
    {
        coordinates(0, 0) = cos(omega1) * dA_AboveX;
        coordinates(0, 1) = sin(omega1) * dA_AboveX;
        coordinates(2, 0) = cos(omega2) * dA_BelowX;
        coordinates(2, 1) = - sin(omega2) * dA_BelowX;
        alphaAngle = omega1;
    }
    else
    {
        coordinates(1, 0) = cos(omega1) * dA_AboveX;
        coordinates(1, 1) = sin(omega1) * dA_AboveX;
        coordinates(3, 0) = cos(omega2) * dA_BelowX;
        coordinates(3, 1) = - sin(omega2) * dA_BelowX;
        alphaAngle = M_PI - (alphaAngle + omega2);
    }

    return coordinates;
}

double SIMphoto::maxPositionDeviation(Matrix<double, 4, 2> system1, Matrix<double, 4, 2> system2)
{
    double maxDeviation = 0.0, newX, difference;

    for(int i = 0; i < 4; i++)
    {
        newX = system2(i, 0) * cos(alphaAngle) + system2(i, 1) * sin(alphaAngle) + system1(0, 0);
        system2(i, 1) = - system2(i, 0) * sin(alphaAngle) + system2(i, 1) * cos(alphaAngle) + system1(0, 1);
        system2(i, 0) = newX;

        difference = sqrt((system1(i, 0) - system2(i, 0)) * (system1(i, 0) - system2(i, 0)) +
                              (system1(i, 1) - system2(i, 1)) * (system1(i, 1) - system2(i, 1)));

        maxDeviation = std::max(maxDeviation, difference);
    }

    return maxDeviation;
}

Matrix<double, 4, 2> SIMphoto::putY_axesToDist4(Matrix<double, 4, 2> system, double dist4)
{
    Matrix<double, 4, 2> newCoordinates;
    double l1, l2;

    l1 = system(0, 1) * dist4 / (system(0, 0) * system(0, 0) + system(0, 1) * system(0, 1));
    l2 = system(0, 0) * dist4 / (system(0, 0) * system(0, 0) + system(0, 1) * system(0, 1));

    for(int i = 0; i < 4; i++)
    {
        newCoordinates(i, 0) = l1 * system(i, 0) - l2 * system(i, 1);
        newCoordinates(i, 1) = l2 * system(i, 0) + l1 * system(i, 1);
    }

    return newCoordinates;
}

void SIMphoto::showDeviationMessage(double dev, Matrix<double, 4, 2> coordsFromNet)
{
    QString text(tr("Position deviation is "));
    text.append(QString::number(dev, 'f', 3)).append(tr(" meters. (More information: SIMphoto manual)"));

    QMessageBox devMsg(this);
    devMsg.setWindowTitle(tr("Information"));
    devMsg.setText(text);

    QAbstractButton *back = devMsg.addButton(tr("Back"), QMessageBox::ActionRole);
    QAbstractButton *agree = devMsg.addButton(tr("I Agree"), QMessageBox::ActionRole);

    devMsg.exec();

    if(devMsg.clickedButton() == back)
        doNetworkMethodAction();

    if(devMsg.clickedButton() == agree)
    {
        disableOtherActions();
        memory.setActionName("networkMethod");
        memory.setNetworkCoordinates(coordsFromNet);
        createMarkingTool(4);
    }
}

void SIMphoto::doDistMethodAction()
{
    disableOtherActions();
    createDistMethodDialog();
}

void SIMphoto::createDistMethodDialog()
{
    distMethodDialog = new DistMethodDialog(this);
    distMethodDialog->show();

    connect(distMethodDialog, SIGNAL(rejected()), this, SLOT(closeDistMethodDialog()));
    connect(distMethodDialog->getOkButton(), SIGNAL(clicked()), this, SLOT(getDistMethod()));
}

void SIMphoto::closeDistMethodDialog()
{
    delete distMethodDialog;
    enableOtherActions();
}

void SIMphoto::getDistMethod()
{
    if(distMethodDialog->getMethod() == 1)
    {
        memory.setActionName("oneDistMethod");
        createMarkingTool(2);
    }
    else
    {
        memory.setActionName("twoDistMethod");
        createMarkingTool(3);
    }

    closeDistMethodDialog();
    disableOtherActions();
}

void SIMphoto::createDistInputDialog()
{
    if(memory.getActionName() == "oneDistMethod")
    {
        distInputDialog = new DistanceInputDialog(1, this);
    }
    else
    {
        distInputDialog = new DistanceInputDialog(2, this);
    }

    distInputDialog->show();

    connect(distInputDialog->getTransformButton(), SIGNAL(clicked()), this, SLOT(getScaleFromDist()));
    connect(distInputDialog, SIGNAL(rejected()), this, SLOT(closeDistInputDialog()));
}

void SIMphoto::closeDistInputDialog()
{
    delete distInputDialog;
}

void SIMphoto::getScaleFromDist()
{
    //checking input parameters...
    if(distInputDialog->getNewScaleSpinBox()->value() < 1e-4)
    {
        QMessageBox::information(this, tr("Warning"), tr("Pixel size cannot be zero!"));
        return;
    }

    if(memory.getActionName() == "oneDistMethod")
    {
        if(distInputDialog->getSepDist1SpinBox()->value() < 0.001)
        {
            QMessageBox::information(this, tr("Warning"), tr("Distance cannot be zero!"));
            return;
        }

        double X1, Y1, X2, Y2, pixelDistance;

        X1 = markingTool->getListOfPoints().at(0)->x() + 11;
        Y1 = markingTool->getListOfPoints().at(0)->y() + 43;
        X2 = markingTool->getListOfPoints().at(1)->x() + 11;
        Y2 = markingTool->getListOfPoints().at(1)->y() + 43;

        pixelDistance = sqrt((X2 - X1) * (X2 - X1) + (Y2 - Y1) * (Y2 - Y1));

        //calculating scale for the similar transformation
        distInputDialog->setInputPixelSize(distInputDialog->getSepDist1SpinBox()->value() / pixelDistance);
        getSimilarTransParameters();
    }
    else
    {
        if(distInputDialog->getSepDist1SpinBox()->value() < 0.001 || distInputDialog->getSepDist2SpinBox()->value() < 0.001)
        {
            QMessageBox::information(this, tr("Warning"), tr("Distance cannot be zero!"));
            return;
        }

        doAffineTransformation();
    }
}

void SIMphoto::doSimilarTransAction()
{
    disableOtherActions();
    memory.setActionName("similarTransAction");
    createSimilarTransDialog();
}

void SIMphoto::createSimilarTransDialog()
{
    similarTransDialog = new SimilarTransformDialog(memory.getPixelSize(), this);
    similarTransDialog->show();

    connect(similarTransDialog, SIGNAL(rotateIsChecked()), this, SLOT(showVerticalDaistance()));
    connect(similarTransDialog, SIGNAL(rotateIsUnchecked()), this, SLOT(hideVerticalDistance()));
    connect(similarTransDialog, SIGNAL(firstPoint()), this, SLOT(setFirstPoint()));
    connect(similarTransDialog, SIGNAL(secondPoint()), this, SLOT(setSecondPoint()));
    connect(similarTransDialog, SIGNAL(rejected()), this, SLOT(closeSimilarTransDialog()));
    connect(similarTransDialog->getTransformButton(), SIGNAL(clicked()), this, SLOT(getSimilarTransParameters()));
}

void SIMphoto::showVerticalDaistance()
{
    markingTool = new MarkTool(2);
    connect(markingTool, SIGNAL(readyToMarkingSignal()), this, SLOT(startMarking()));

    markingTool->getStartButton()->click();
}

void SIMphoto::hideVerticalDistance()
{
    if(zoomIsActive)
        zoomLabel->hide();

    markingTool->close();
    delete markingTool;
    markingTool = 0;

    imageScrollArea->widget()->setCursor(Qt::ArrowCursor);

    if(overviewLabel)
        overviewLabel->setPixmap(QPixmap::fromImage(image->scaled(400, 400, Qt::KeepAspectRatio, Qt::FastTransformation)));

    imageLabel->setPixmap(QPixmap::fromImage(*image));
    imageLabel->adjustSize();
}

void SIMphoto::closeSimilarTransDialog()
{
    if(markingTool)
        hideVerticalDistance();

    delete similarTransDialog;

    enableOtherActions();
}

//calculating rotation angle and scale for similar transformation
void SIMphoto::getSimilarTransParameters()
{
    double scale = 1.0;
    double angle = 0.0;

    if(memory.getActionName() == "oneDistMethod")
    {
        scale = distInputDialog->getInputPixelSize() / distInputDialog->getNewScaleSpinBox()->value();

        if(distInputDialog->getRotateCheckBox()->isChecked())
        {
            angle = specifyQuadrant(markingTool->getListOfPoints().at(0)->x() + 11, markingTool->getListOfPoints().at(0)->y() + 43,
                                    markingTool->getListOfPoints().at(1)->x() + 11, markingTool->getListOfPoints().at(1)->y() + 43);
        }

        memory.setPreviousPixelSize(memory.getPixelSize());
        memory.setPixelSize(distInputDialog->getNewScaleSpinBox()->value());
        memory.setPreviousImage(image);
    }
    else
    {
        if(similarTransDialog->getChangeScaleCheckBox()->isChecked())
        {
            if(memory.getPixelSize() < 1e-4)
            {
                scale = similarTransDialog->getPercentageScaleSpinBox()->value() / 100.0;
            }
            else
            {
                scale = memory.getPixelSize() / similarTransDialog->getNewScaleSpinBox()->value();

                //If the size of the output image is twice bigger, transformation will be breaked.
                if (scale > 2)
                {
                    QMessageBox::information(this, tr("Too large scale!"), tr("The scale is more than twice bigger!"));
                    return;
                }
            }
        }

        if(similarTransDialog->getRotateCheckBox()->isChecked())
        {
            angle = specifyQuadrant(markingTool->getListOfPoints().at(0)->x() + 11, markingTool->getListOfPoints().at(0)->y() + 43,
                                    markingTool->getListOfPoints().at(1)->x() + 11, markingTool->getListOfPoints().at(1)->y() + 43);
        }

        memory.setPreviousPixelSize(memory.getPixelSize());
        memory.setPixelSize(similarTransDialog->getNewScaleSpinBox()->value());
        memory.setPreviousImage(image);
    }

    doSimilarTransformation(scale, angle);
}

void SIMphoto::doSimilarTransformation(double scaleForTrans, double rotationAngle)
{
    double lambda1, lambda2;

    //saving the current state
    previousCornersCoords = cornersCoords;

    QSize newImageSize;
    newImageSize = newImageSizeForSimTrans(scaleForTrans, rotationAngle);

    image->~QImage();
    image = new QImage(newImageSize.width(), newImageSize.height(), QImage::Format_ARGB32);

    memory.setPreviousObjectCoordsKey(memory.getObjectCoordsKey());
    memory.setPreviousCornersCrd(memory.getCornersCrd());

    if(memory.getActionName() == "oneDistMethod")
    {
        memory.setCornersCrd(false);
        statusBar()->clearMessage();
    }
    else
    {
        if(memory.getCornersCrd())
        {
            memory.setCornersCrd(true);
            CoeffForObjCrd();
        }
        else
        {
            memory.setCornersCrd(false);
            statusBar()->clearMessage();
        }
    }

    //resampling...
    QRgb black = qRgba(0, 0, 0, 255);
    image->fill(black);

    int indexX, indexY, red, green, blue;
    double alpha, beta, transformX, transformY;
    QRgb rgbA, rgbB, rgbC, rgbD, newRgb;

    QProgressDialog progress(tr("Resampling image..."), tr("Abort Resampling"), 0, newImageSize.width(), this);
    progress.setWindowModality(Qt::WindowModal);
    progress.setMinimumDuration(0);

    bool progressAborted = false;

    lambda1 = cos(rotationAngle) / scaleForTrans;
    lambda2 = sin(rotationAngle) / scaleForTrans;

    for(int i = 0; i < newImageSize.width(); i++)
    {
        progress.setValue(i);

        if(progress.wasCanceled())
        {
            imageWasModified();
            undo();

            if(markingTool)
                drawDistLines();

            if(overviewLabel)
                drawOverview();

            progressAborted = true;

            break;
        }

        for(int j = 0; j < newImageSize.height(); j++)
        {
            transformX = lambda1 * (i - translationX) + lambda2 * (j - translationY);
            transformY = - lambda2 * (i - translationX) + lambda1 * (j - translationY);

            indexX = (int) floor(transformX);
            indexY = (int) floor(transformY);

            if((indexX >= 1 && indexX < memory.getPreviousImage()->width() - 1) && (indexY >= 1 && indexY < memory.getPreviousImage()->height() - 1))
            {
                alpha = transformX - indexX;
                beta = transformY - indexY;

                rgbA = memory.getPreviousImage()->pixel(indexX, indexY);
                rgbB = memory.getPreviousImage()->pixel(indexX + 1, indexY);
                rgbC = memory.getPreviousImage()->pixel(indexX, indexY + 1);
                rgbD = memory.getPreviousImage()->pixel(indexX + 1, indexY + 1);

                red = (int) round((1 - beta) * ((1 - alpha) * qRed(rgbA) +  alpha * qRed(rgbB)) + beta * ((1 - alpha) * qRed(rgbC) + alpha * qRed(rgbD)));
                green = (int) round((1 - beta) * ((1 - alpha) * qGreen(rgbA) +  alpha * qGreen(rgbB)) + beta * ((1 - alpha) * qGreen(rgbC) + alpha * qGreen(rgbD)));
                blue = (int) round((1 - beta) * ((1 - alpha) * qBlue(rgbA) +  alpha * qBlue(rgbB)) + beta * ((1 - alpha) * qBlue(rgbC) + alpha * qBlue(rgbD)));

                newRgb = qRgba(red, green, blue, 255);

                image->setPixel(i, j, newRgb);
            }
        }
    }

    progress.setValue(newImageSize.width());

    if(!progressAborted)
    {
        if(memory.getActionName() == "oneDistMethod")
        {
            delete distInputDialog;
            closeMarkingTool();
        }
        else
        {
            closeSimilarTransDialog();
        }

        imageLabel->setPixmap(QPixmap::fromImage(*image));
        imageLabel->adjustSize();

        if(overviewLabel)
            overviewLabel->setOverview(image, imageScrollArea->size(), imageScrollArea->widget()->pos());

        imageWasModified();
    }
}

void SIMphoto::doAffineTransformation()
{
    double verticalMeters, verticalPixels;
    double horizontalMeters, horizontalPixels;
    double pixelHeight, pixelWidth;
    double mx = 1.0, my = 1.0;
    double pixelSize = 0.0;

    //calculating scales mx, my
    verticalMeters = distInputDialog->getSepDist1SpinBox()->value();
    horizontalMeters = distInputDialog->getSepDist2SpinBox()->value();

    verticalPixels = distanceFromCoords(markingTool->getListOfPoints().at(0)->x() + 11,
                                       markingTool->getListOfPoints().at(0)->y() + 43,
                                       markingTool->getListOfPoints().at(1)->x() + 11,
                                       markingTool->getListOfPoints().at(1)->y() + 43);

    horizontalPixels = distanceFromCoords(markingTool->getListOfPoints().at(1)->x() + 11,
                                         markingTool->getListOfPoints().at(1)->y() + 43,
                                         markingTool->getListOfPoints().at(2)->x() + 11,
                                         markingTool->getListOfPoints().at(2)->y() + 43);

    pixelHeight = verticalMeters / verticalPixels;
    pixelWidth = horizontalMeters / horizontalPixels;

    if(pixelHeight >=  pixelWidth)
    {
        my = pixelHeight /  pixelWidth;
        pixelSize = pixelWidth;
    }
    if(pixelHeight < pixelWidth)
    {
        mx =  pixelWidth / pixelHeight;
        pixelSize = pixelHeight;
    }

    double scaleCoeff;

    scaleCoeff = pixelSize / distInputDialog->getNewScaleSpinBox()->value();

    mx *= scaleCoeff;
    my *= scaleCoeff;

    //saving current state
    previousCornersCoords = cornersCoords;
    memory.setPreviousObjectCoordsKey(memory.getObjectCoordsKey());
    memory.setPreviousCornersCrd(memory.getCornersCrd());
    memory.setCornersCrd(false);
    statusBar()->clearMessage();

    memory.setPreviousPixelSize(memory.getPixelSize());
    memory.setPixelSize(distInputDialog->getNewScaleSpinBox()->value());
    memory.setPreviousImage(image);

    QSize newImageSize;
    newImageSize.setWidth((int)(mx * image->width() + 1));
    newImageSize.setHeight((int)(my * image->height() + 1));

    image->~QImage();
    image = new QImage(newImageSize.width(), newImageSize.height(), QImage::Format_ARGB32);

    //resampling
    QRgb black = qRgba(0, 0, 0, 255);
    image->fill(black);

    int indexX, indexY, red, green, blue;
    double alpha, beta, transformX, transformY;
    QRgb rgbA, rgbB, rgbC, rgbD, newRgb;

    QProgressDialog progress(tr("Resampling image..."), tr("Abort Resampling"), 0, newImageSize.width(), this);
    progress.setWindowModality(Qt::WindowModal);
    progress.setMinimumDuration(0);

    bool progressAborted = false;

    for(int i = 0; i < newImageSize.width(); i++)
    {
        progress.setValue(i);

        if(progress.wasCanceled())
        {
            imageWasModified();
            undo();
            drawDistLines();

            if(overviewLabel)
                drawOverview();

            progressAborted = true;

            break;
        }

        for(int j = 0; j < newImageSize.height(); j++)
        {
            transformX = i / mx;
            transformY = j / my;

            indexX = (int) floor(transformX);
            indexY = (int) floor(transformY);

            if((indexX >= 1 && indexX < memory.getPreviousImage()->width() - 1) && (indexY >= 1 && indexY < memory.getPreviousImage()->height() - 1))
            {
                alpha = transformX - indexX;
                beta = transformY - indexY;

                rgbA = memory.getPreviousImage()->pixel(indexX, indexY);
                rgbB = memory.getPreviousImage()->pixel(indexX + 1, indexY);
                rgbC = memory.getPreviousImage()->pixel(indexX, indexY + 1);
                rgbD = memory.getPreviousImage()->pixel(indexX + 1, indexY + 1);

                red = (int) round((1 - beta) * ((1 - alpha) * qRed(rgbA) +  alpha * qRed(rgbB)) + beta * ((1 - alpha) * qRed(rgbC) + alpha * qRed(rgbD)));
                green = (int) round((1 - beta) * ((1 - alpha) * qGreen(rgbA) +  alpha * qGreen(rgbB)) + beta * ((1 - alpha) * qGreen(rgbC) + alpha * qGreen(rgbD)));
                blue = (int) round((1 - beta) * ((1 - alpha) * qBlue(rgbA) +  alpha * qBlue(rgbB)) + beta * ((1 - alpha) * qBlue(rgbC) + alpha * qBlue(rgbD)));

                newRgb = qRgba(red, green, blue, 255);

                image->setPixel(i, j, newRgb);
            }
        }
    }

    progress.setValue(newImageSize.width());

    if(!progressAborted)
    {
        delete distInputDialog;
        closeMarkingTool();

        imageLabel->setPixmap(QPixmap::fromImage(*image));
        imageLabel->adjustSize();

        if(overviewLabel)
            overviewLabel->setOverview(image, imageScrollArea->size(), imageScrollArea->widget()->pos());

        imageWasModified();
    }
}

double SIMphoto::distanceFromCoords(double XA, double YA, double XB, double YB)
{
    double d;

    d = sqrt((XA - XB) * (XA - XB) + (YA - YB) * (YA - YB));
    return d;
}

//Function determines quadrant and computes rotation angle for similar transformation
double SIMphoto::specifyQuadrant(int topX, int topY, int bottomX, int bottomY)
{
    double omega = 0.0;

    if (topX < bottomX && topY < bottomY)
    {
        omega = atan(1.0 * (bottomX - topX) / (bottomY - topY));
    }
    if (topX > bottomX && topY < bottomY)
    {
        omega = 2 * M_PI - atan(1.0 * abs(bottomX - topX) / (bottomY - topY));
    }
    if (topX > bottomX && topY > bottomY)
    {
        omega = M_PI + atan(1.0 * (bottomX - topX) / (bottomY - topY));
     }
    if (topX < bottomX && topY > bottomY)
    {
        omega = M_PI - atan(1.0 * (bottomX - topX) / abs(bottomY - topY));
    }
    if (topX < bottomX && topY == bottomY)
    {
        omega = M_PI / 2.0;
    }
    if (topX > bottomX && topY == bottomY)
    {
        omega = 3.0 * M_PI / 2.0;
    }
    if (topX == bottomX && topY < bottomY)
    {
        omega = 0.0;
    }
    if (topX == bottomX && topY > bottomY)
    {
        omega = M_PI;
    }
    return omega;
}

//Befor similar transformation new raster size is computed
QSize SIMphoto::newImageSizeForSimTrans(double transScale, double transAngle)
{
    QSize newSize;
    double l1, l2;
    double minX, minY, maxX, maxY;

    l1 = transScale * cos(transAngle);
    l2 = transScale * sin(transAngle);

    Matrix<double, 4, 2> imageCoord, corner;

    //coordinates of input raster
    imageCoord << 0, 0,
                  image->width() - 1, 0,
                  image->width() - 1, image->height() - 1,
                  0, image->height() - 1;

    for (int i = 0; i < 4; i++)
    {
        //transform equations
        corner(i, 0) = imageCoord(i, 0) * l1 - imageCoord(i, 1) * l2;
        corner(i, 1) = imageCoord(i, 0) * l2 + imageCoord(i, 1) * l1;

        //Min & max coordinates are computed for determining translation
        //left corner of a new image has coordinates (0, 0)
        if (i == 0)
        {
            minX = corner(i, 0);
            minY = corner(i, 1);
            maxX = minX;
            maxY = minY;
        }
        else
        {
            minX = std::min(minX, corner(i, 0));
            minY = std::min(minY, corner(i, 1));
            maxX = std::max(maxX, corner(i, 0));
            maxY = std::max(maxY, corner(i, 1));
        }

    }

    translationX = - minX;
    translationY = - minY;

    if(memory.getCornersCrd() && memory.getActionName() == "similarTransAction")
    {
        //calculating new geodetic coordinates of image corners
        double x1Obj, y1Obj, x2Obj, y2Obj;
        Matrix<double, 2, 1> aa;
        Matrix<double, 2, 1> XAB;
        Matrix<double, 2, 2> xab;

        XAB << cornersCoords(1, 0) - cornersCoords(0, 0),
               cornersCoords(0, 1) - cornersCoords(1, 1);

        xab << corner(2, 0) - corner(0, 0), - (corner(2, 1) - corner(0, 1)),
               corner(2, 1) - corner(0, 1), corner(2, 0) - corner(0, 0);

        aa = xab.inverse() * XAB;

        x1Obj = cornersCoords(0, 0) + aa(0) * minX - aa(1) * minY;
        y1Obj = cornersCoords(0, 1) - aa(0) * minY - aa(1) * minX;
        x2Obj = cornersCoords(0, 0) + aa(0) * maxX - aa(1) * maxY;
        y2Obj = cornersCoords(0, 1) - aa(0) * maxY - aa(1) * maxX;

        cornersCoords(0, 0) = x1Obj;
        cornersCoords(0, 1) = y1Obj;
        cornersCoords(1, 0) = x2Obj;
        cornersCoords(1, 1) = y2Obj;
    }

    newSize.setWidth((int) round(maxX - minX + 1));
    newSize.setHeight((int) round(maxY - minY + 1));

    return newSize;
}

void SIMphoto::doDistorsionRedAction()
{
    disableOtherActions();
    createDistortionDialog();
}

void SIMphoto::createDistortionDialog()
{
    distorsionDialog = new DistorsionDialog(this);
    distorsionDialog->show();

    connect(distorsionDialog, SIGNAL(rejected()), this, SLOT(closeDistortionDialog()));
    connect(distorsionDialog->getButton(), SIGNAL(clicked()), this, SLOT(checkDistortionInput()));
}

void SIMphoto::checkDistortionInput()
{
    if(distorsionDialog->getW()->value() < 1e-4 || distorsionDialog->getH()->value() < 1e-4)
    {
        QMessageBox::information(this, tr("Warning"), tr("Format size cannot be zero!"));
        return;
    }

    if(distorsionDialog->getPPX()->value() < 1e-4 || distorsionDialog->getPPY()->value() < 1e-4)
    {
        QMessageBox::information(this, tr("Warning"), tr("Coordinates of the principal point cannot be zero!"));
        return;
    }

    doDistortionReduction();
}

void SIMphoto::doDistortionReduction()
{
    //pixel = pixel size [mm]; pp = principal point; K,P = distortion coeffs.; r = radial distace from pp
    double pixel, ppX, ppY, K1, K2, K3, P1, P2, r;
    int w, h;

    w = image->width();
    h = image->height();

    Matrix<double, 3, Eigen::Dynamic> meterXY;
    Matrix<double, 2, 1> meterXY2;
    Matrix<double, 3, Eigen::Dynamic> meterXY0;
    Matrix<double, 2, 1> dRadial;
    Matrix<double, 2, 1> dTangencial;
    Matrix<double, 3, 3> ppReductionMatrix;

    pixel = (distorsionDialog->getW()->value() / w + distorsionDialog->getH()->value() / h) / 2.0;
    ppX = distorsionDialog->getPPX()->value();
    ppY = distorsionDialog->getPPY()->value();
    K1 = distorsionDialog->getK1()->value();
    K2 = distorsionDialog->getK2()->value();
    K3 = distorsionDialog->getK3()->value();
    P1 = distorsionDialog->getP1()->value();
    P2 = distorsionDialog->getP2()->value();

    meterXY.resize(3, h);
    meterXY0.resize(3, h);

    //saving the current state
    previousCornersCoords = cornersCoords;

    memory.setPreviousObjectCoordsKey(memory.getObjectCoordsKey());
    memory.setPreviousCornersCrd(memory.getCornersCrd());

    memory.setCornersCrd(false);
    statusBar()->clearMessage();

    memory.setPreviousImage(image);

    image->~QImage();
    image = new QImage(w, h, QImage::Format_ARGB32);

    //resampling...
    QRgb black = qRgba(0, 0, 0, 255);
    image->fill(black);

    int indexX, indexY, red, green, blue;
    double alpha, beta, transformX, transformY;
    QRgb rgbA, rgbB, rgbC, rgbD, newRgb;

    QProgressDialog progress(tr("Calculating distortion residuals..."), tr("Abort Calculating"), 0, w, this);
    progress.setWindowModality(Qt::WindowModal);
    progress.setMinimumDuration(0);

    bool progressAborted = false;

    for(int wi = 0; wi < w; wi++)
    {
        progress.setValue(wi);

        if(progress.wasCanceled())
        {
            imageWasModified();
            undo();

            if(overviewLabel)
                drawOverview();

            progressAborted = true;

            break;
        }

        if(!progressAborted)
        {
            //matrix that transform raster coordinates to coordinates of principle point
            ppReductionMatrix << pixel, 0.0, - ppX + 0.5 * pixel,
                                 0.0, - pixel, - ppY + 0.5 * pixel + h * pixel,
                                 0.0, 0.0, 1.0;

            //creating raster coordinates for each pixel
            for(int i = 0; i < h; i++)
            {
                    meterXY(0, i) = wi + 1;
                    meterXY(1, i) = i + 1;
                    meterXY(2, i) = 1;
            }

            //transforming to principle point system
            meterXY = ppReductionMatrix * meterXY;

            for(int i = 0; i < h; i++)
            {
                //radial distance
                r = sqrt(meterXY(0, i) * meterXY(0, i) + meterXY(1, i) * meterXY(1, i));

                //calculating radial part
                dRadial(0) = meterXY(0, i) * (K1 * r * r + K2 * r * r * r * r + K3 * r * r * r * r * r * r);
                dRadial(1) = meterXY(1, i) * (K1 * r * r + K2 * r * r * r * r + K3 * r * r * r * r * r * r);

                //calculating tangencial part
                dTangencial(0) = P1 * (r * r + 2 * meterXY(0, i) * meterXY(0, i)) + 2 * P2 * meterXY(0, i) * meterXY(1, i);
                dTangencial(1) = P2 * (r * r + 2 * meterXY(1, i) * meterXY(1, i)) + 2 * P1 * meterXY(0, i) * meterXY(1, i);

                //correction
                meterXY2(0) = meterXY(0, i) + dRadial(0) + dTangencial(0);
                meterXY2(1) = meterXY(1, i) + dRadial(1) + dTangencial(1);

                meterXY0(0, i) = 2 * meterXY(0, i) - meterXY2(0);
                meterXY0(1, i) = 2 * meterXY(1, i) - meterXY2(1);
                meterXY0(2, i) = 1;

                //finding inverse expression for the pixel
                for(int j = 0; j < 4; j++)
                {
                    r = sqrt(meterXY0(0, i) * meterXY0(0, i) + meterXY0(1, i) * meterXY0(1, i));

                    dRadial(0) = meterXY0(0, i) * (K1 * r * r + K2 * r * r * r * r + K3 * r * r * r * r * r * r);
                    dRadial(1) = meterXY0(1, i) * (K1 * r * r + K2 * r * r * r * r + K3 * r * r * r * r * r * r);

                    dTangencial(0) = P1 * (r * r + 2 * meterXY0(0, i) * meterXY0(0, i)) + 2 * P2 * meterXY0(0, i) * meterXY0(1, i);
                    dTangencial(1) = P2 * (r * r + 2 * meterXY0(1, i) * meterXY0(1, i)) + 2 * P1 * meterXY0(0, i) * meterXY0(1, i);

                    meterXY2(0) = meterXY0(0, i) + dRadial(0) + dTangencial(0);
                    meterXY2(1) = meterXY0(1, i) + dRadial(1) + dTangencial(1);

                    meterXY0(0, i) = meterXY0(0, i) + meterXY(0, i) - meterXY2(0);
                    meterXY0(1, i) = meterXY0(1, i) + meterXY(1, i) - meterXY2(1);
                }
            }

            //transformation from principal point system to raster coordinates
            meterXY = ppReductionMatrix.inverse() * meterXY0;

            for(int i = 0; i < h; i++)
            {
                transformX = meterXY(0, i);
                transformY = meterXY(1, i);

                indexX = (int) floor(transformX);
                indexY = (int) floor(transformY);

                if((indexX >= 1 && indexX < memory.getPreviousImage()->width() - 1) && (indexY >= 1 && indexY < memory.getPreviousImage()->height() - 1))
                {
                    alpha = transformX - indexX;
                    beta = transformY - indexY;

                    rgbA = memory.getPreviousImage()->pixel(indexX, indexY);
                    rgbB = memory.getPreviousImage()->pixel(indexX + 1, indexY);
                    rgbC = memory.getPreviousImage()->pixel(indexX, indexY + 1);
                    rgbD = memory.getPreviousImage()->pixel(indexX + 1, indexY + 1);

                    red = (int) round((1 - beta) * ((1 - alpha) * qRed(rgbA) +  alpha * qRed(rgbB)) + beta * ((1 - alpha) * qRed(rgbC) + alpha * qRed(rgbD)));
                    green = (int) round((1 - beta) * ((1 - alpha) * qGreen(rgbA) +  alpha * qGreen(rgbB)) + beta * ((1 - alpha) * qGreen(rgbC) + alpha * qGreen(rgbD)));
                    blue = (int) round((1 - beta) * ((1 - alpha) * qBlue(rgbA) +  alpha * qBlue(rgbB)) + beta * ((1 - alpha) * qBlue(rgbC) + alpha * qBlue(rgbD)));

                    newRgb = qRgba(red, green, blue, 255);

                    image->setPixel(wi, i, newRgb);
                }
            }
        }
    }

    if(!progressAborted)
    {
        closeDistortionDialog();

        imageLabel->setPixmap(QPixmap::fromImage(*image));
        imageLabel->adjustSize();

        if(overviewLabel)
            overviewLabel->setOverview(image, imageScrollArea->size(), imageScrollArea->widget()->pos());

        imageWasModified();
    }
}

void SIMphoto::closeDistortionDialog()
{
    delete distorsionDialog;
    enableOtherActions();
}

void SIMphoto::doLoadTableAction()
{
    QString path;
    getInitPath(path);

    QString fileName = QFileDialog::getOpenFileName(this,
                                     tr("Open File"), path,
                                     "TXT (*.txt *.TXT)");

    QString line;
    QFile file(fileName);

    //specifying text separator
    QRegExp sep("\\s+");
    int i = -1;

    QList<QString> residualXList;
    QList<QString> residualYList;

    Matrix<int, 9, 1> usePoint;
    Matrix<double, 9, 2> objCoords;
    Matrix<int, 9, 2> imCoords;
    usePoint.setOnes();
    objCoords.setZero();

    if(file.open(QIODevice::ReadOnly))
    {
        QTextStream in(&file);
        while(!in.atEnd())
        {
            line = in.readLine();

            //testing header validity
            if(i == -1)
            {
                QString headerStr(tr("UsePointX[m]Y[m]x[p]y[p]ResidualX[m]ResidualY[m]"));
                QString testStr;

                testStr.append(line.section(sep, 0, 0)).append(line.section(sep, 1, 1)).append(line.section(sep, 2, 2))
                        .append(line.section(sep, 3, 3)).append(line.section(sep, 4, 4)).append(line.section(sep, 5, 5))
                        .append(line.section(sep, 6, 6)).append(line.section(sep, 7, 7)).append(line.section(sep, 8, 8))
                        .append(line.section(sep, 9, 9));

                if(testStr != headerStr)
                {
                    QMessageBox::information(this, tr("Warning!"), tr("Invalid input file! Make sure the file contains the correct header."));
                    return;
                }
            }
            else
            {
                //saving data from txt file
                objCoords(i, 0) = line.section(sep, 2, 2).toDouble();
                objCoords(i, 1) = line.section(sep, 3, 3).toDouble();

                imCoords(i, 0) = line.section(sep, 4, 4).toInt();
                imCoords(i, 1) = line.section(sep, 5, 5).toInt();

                if(line.at(0) == 'N')
                    usePoint(i) = 0;

                residualXList.append(line.section(sep, 6, 6));
                residualYList.append(line.section(sep, 7, 7));
            }

            i++;

            if(i == 9)
                break;
        }

        if(i > 0)
        {
            //creation of marking tool
            disableOtherActions();
            memory.setActionName("controlPointsMethod");
            createMarkingTool(9);
            markingTool->getStartButton()->click();

            movePoint(imCoords(0, 0) - 11, imCoords(0, 1) - 43);

            for(int j = 1; j < i; j++)
            {
                markingTool->getSpinBox()->setValue(j + 1);
                movePoint(imCoords(j, 0) - 11, imCoords(j, 1) - 43);
            }

            memory.setObjectCoordinates(objCoords);
            showExistingTable(residualXList, residualYList, usePoint);
        }
    }
}

//creating table of points
void SIMphoto::showExistingTable(QList<QString> residualXList, QList<QString> residualYList, Matrix<int, 9, 1> usePoint)
{

    createTableOfPoints();

    QDoubleSpinBox residual;
    residual.setDecimals(4);
    residual.setRange(-1000, 1000);

    for(int i = 0; i < markingTool->getPointsCount(); i++)
    {
        if(usePoint(i) == 0)
            tableOfPoints->getCheckBoxList().at(i)->setChecked(false);

        if(!residualXList.at(i).isEmpty() && !residualYList.at(i).isEmpty() )
        {
            tableOfPoints->getTable()->item(i, 6)->setData(Qt::DisplayRole, residualXList.at(i).toDouble());
            tableOfPoints->getTable()->item(i, 6)->setFont(QFont("Arial", 9, QFont::Bold));
            tableOfPoints->getTable()->item(i, 7)->setData(Qt::DisplayRole, residualYList.at(i).toDouble());
            tableOfPoints->getTable()->item(i, 7)->setFont(QFont("Arial", 9, QFont::Bold));
        }
    }
}

void SIMphoto::doMeasureAction()
{
    disableOtherActions();
    memory.setActionName("measureAction");
    createMeasureTool();
}

void SIMphoto::createMeasureTool()
{
    markingTool = new MarkTool(2);
    measureTool = new MeasureTool;

    markDock = new QDockWidget(tr("Measurement"), this);
    markDock->setAllowedAreas(Qt::LeftDockWidgetArea | Qt::RightDockWidgetArea);
    markDock->setFeatures(QDockWidget::DockWidgetMovable | QDockWidget::DockWidgetFloatable);
    markDock->setWidget(measureTool);

    addDockWidget(Qt::LeftDockWidgetArea, markDock);

    connect(measureTool, SIGNAL(firstPoint()), this, SLOT(setFirstPoint()));
    connect(measureTool, SIGNAL(secondPoint()), this, SLOT(setSecondPoint()));
    connect(measureTool->getExitButton(), SIGNAL(clicked()), this, SLOT(closeMeasureAction()));
    connect(markingTool, SIGNAL(readyToMarkingSignal()), this, SLOT(startMarking()));

    markingTool->getStartButton()->click();
}

void SIMphoto::setFirstPoint()
{
    markingTool->getSpinBox()->setValue(1);
}

void SIMphoto::setSecondPoint()
{
    markingTool->getSpinBox()->setValue(2);
}

void SIMphoto::closeMeasureAction()
{
    if(zoomIsActive)
        zoomLabel->hide();

    delete measureTool;
    closeMarkingTool();
}

void SIMphoto::doScalebarAction()
{
    disableOtherActions();
    memory.setActionName("scalebarAction");
    finishScalebarAction = false;
    setCrossCursor();
    createScalebarDialog();
}

void SIMphoto::createScalebarDialog()
{
    scalebarDialog = new ScalebarDialog(memory.getPixelSize(), this);
    scalebarDialog->show();

    connect(scalebarDialog, SIGNAL(rejected()), this, SLOT(closeScalebarDialog()));
    connect(scalebarDialog->getFinishButton(), SIGNAL(clicked()), this, SLOT(finishScalebar()));
}

void SIMphoto::closeScalebarDialog()
{
    imageScrollArea->widget()->setCursor(Qt::ArrowCursor);

    if(finishScalebarAction == false)
    {
        imageLabel->setPixmap(QPixmap::fromImage(*image));
        imageLabel->adjustSize();
    }

    if(overviewLabel)
        overviewLabel->setPixmap(QPixmap::fromImage(image->scaled(400, 400, Qt::KeepAspectRatio, Qt::FastTransformation)));

    memory.setActionName("noAction");
    enableOtherActions();
}

//function for completing scale bar
void SIMphoto::drawScalebar(int x, int y)
{
    int scalebarWidth = 0;
    int scalebarSize;
    double scalebarValue = 1e-4;

    //getting size of scale bar
    setScalebarValue(scalebarWidth, scalebarValue);

    //coordinates of segments
    Matrix<int, 7, 1> segment;
    segment << x + 60,
               x + 60 + (int)round(scalebarWidth / 10.0),
               x + 60 + (int)round(scalebarWidth / 5.0),
               x + 60 + (int)round(2.0 * scalebarWidth / 5.0),
               x + 60 + (int)round(3.0 * scalebarWidth / 5.0),
               x + 60 + (int)round(4.0 * scalebarWidth / 5.0),
               x + 60 + scalebarWidth;

    scalebarSize = scalebarDialog->getScaleSizeBox()->value();


    QImage *drawing = new QImage(*image);
    QString value;
    value.append(QString::number(scalebarValue));

    QPainter *painter = new QPainter(drawing);
    painter->setPen(QPen(QColor(0, 0, 0), scalebarSize / 10, Qt::SolidLine,Qt::FlatCap, Qt::MiterJoin));

    QBrush brush(QColor(255, 255, 255), Qt::SolidPattern);
    painter->setBrush(brush);
    painter->setRenderHint(QPainter::Antialiasing, true);

    value = QString::number(5 * scalebarValue);
    value.append(" ").append(scalebarDialog->getScaleUnitsComboBox()->currentText());

    scalebarRect = QRect(x, y, 60 + scalebarWidth + scalebarSize * value.size(), 40 + 3 * scalebarSize);

    //drawing scale bar frame
    painter->drawRect(scalebarRect);
    painter->drawRect(x + 60, y + 2 * scalebarSize + 20, scalebarWidth, scalebarSize);

    //drawing scale bar segments
    brush.setColor(QColor(0, 0, 0));
    painter->setBrush(brush);
    painter->drawRect(segment(1), y + 2 * scalebarSize + 20, segment(2) - segment(1), scalebarSize);
    painter->drawRect(segment(3), y + 2 * scalebarSize + 20, segment(4) - segment(3), scalebarSize);
    painter->drawRect(segment(5), y + 2 * scalebarSize + 20, segment(6) - segment(5), scalebarSize);

    painter->setPen(QPen(QColor(0, 0, 0), scalebarSize / 10, Qt::SolidLine,Qt::FlatCap, Qt::MiterJoin));

    for(int i = 0; i < 7; i++)
    {
        painter->drawLine(segment(i), y + scalebarSize + 20, segment(i), y + 2 * scalebarSize + 20);
    }

    //drawing zero
    painter->setFont(QFont("Arial", scalebarSize, QFont::Bold));
    painter->drawText(x + 60, y + scalebarSize + 15, "0");
    value = QString::number(scalebarValue / 2.0, 'f', 5);

    //removing zeros after decimal number
    for(int i = 0; i < 6; i++)
    {
        if(value.right(1) == "0" || value.right(1) == ".")
        {
            value.remove(value.size() - 1, 1);
        }
        else
        {
            break;
        }
    }

    //drawing text
    for(int i = 1; i < 6; i++)
    {
        painter->drawText(segment(i), y + scalebarSize + 15, value);
        value = QString::number(i * scalebarValue);
    }

    value.append(" ").append(scalebarDialog->getScaleUnitsComboBox()->currentText());
    painter->drawText(x + 60 + scalebarWidth, y + scalebarSize + 15, value);

    if(finishScalebarAction)
    {
        image->~QImage();
        image = new QImage(*drawing);
        imageLabel->setPixmap(QPixmap::fromImage(*image));
        imageLabel->adjustSize();
    }
    else
    {
        imageLabel->setPixmap(QPixmap::fromImage(*drawing));
        imageLabel->adjustSize();
    }

    painter->end();

    delete drawing;
    delete painter;

    if(overviewLabel)
        drawOverview();

    scalebarDialog->getFinishButton()->setEnabled(true);
}

//computing apropriante segment size
void SIMphoto::setScalebarValue(int &scaleWidth, double &scaleValue)
{
    double hundredPix;
    double minimum = 100000.0;
    double units = 0.0;
    double result = 0.0;
    double factor;

    factor = scalebarDialog->getScaleFactorComboBox()->currentText().toDouble();

    //appropriate segment size is about 100 pixels
    hundredPix = memory.getPixelSize() * 100;

    //getting units
    if(scalebarDialog->getScaleUnitsComboBox()->currentIndex() == 0)
        units = 100.0;
    if(scalebarDialog->getScaleUnitsComboBox()->currentIndex() == 1)
        units = 10.0;
    if(scalebarDialog->getScaleUnitsComboBox()->currentIndex() == 2)
        units = 1.0;
    if(scalebarDialog->getScaleUnitsComboBox()->currentIndex() == 3)
         units = 0.001;

    hundredPix *= units;

    //finding minimum difference between segment length and 100 pixels
    for(int i = 0; i < 12; i++)
    {
        if(fabs(scaleValue - hundredPix) < minimum)
        {
            minimum = fabs(scaleValue - hundredPix);
            result = scaleValue;
        }

        scaleValue *= 10;
    }

    scaleValue = result * factor;
    scaleWidth = (int) round(5 * scaleValue / units / memory.getPixelSize());
}

//painting scale bar into the image
void SIMphoto::finishScalebar()
{
    //saving the current state
    finishScalebarAction = true;
    previousCornersCoords = cornersCoords;
    memory.setPreviousImage(image);
    memory.setPreviousObjectCoordsKey(memory.getObjectCoordsKey());
    memory.setPreviousCornersCrd(memory.getCornersCrd());
    memory.setPreviousPixelSize(memory.getPixelSize());

    memory.setCornersCrd(true);
    CoeffForObjCrd();

    drawScalebar(scalebarRect.x(), scalebarRect.y());
    closeScalebarDialog();

    delete scalebarDialog;
    imageWasModified();
}

void SIMphoto::doOverviewAction()
{
    if(overviewLabel)
    {
        delete overviewLabel;
        delete overviewDock;

        overviewLabel = 0;
        overviewDock = 0;
    }
    else
    {
        createOverview();
    }
}

void SIMphoto::doCursorColorAction()
{
    cursorColorDialog = new CursorColorDialog(this);
    cursorColorDialog->show();

    connect(cursorColorDialog, SIGNAL(colorChanged()), this, SLOT(setCursorColor()));
}

void SIMphoto::doZoom3xAction()
{
    if(zoomIsActive)
        zoomLabel->hide();

    zoomInt = 3;
}

void SIMphoto::doZoom5xAction()
{
    if(zoomIsActive)
        zoomLabel->hide();

    zoomInt = 5;
}

void SIMphoto::doNoZoomAction()
{
    if(zoomIsActive)
        zoomLabel->hide();

    zoomInt = 1;
}

//specifying which object recieves which event
bool SIMphoto::eventFilter(QObject *obj, QEvent *event)
{
    if(obj == imageScrollArea && overviewLabel)
    {
        if(event->type() == QEvent::Resize)
        {
            //setting overview frame size by resize event
            setFramePos();
        }
        else
        {
            return false;
        }
    }
    if(obj == overviewLabel)
    {
        if(event->type() == QEvent::MouseMove)
        {
            //frame move by mouse move
            QMouseEvent *overviewMoveEvent = static_cast<QMouseEvent*>(event);
            doOverviewMoveEvent(overviewMoveEvent->x(), overviewMoveEvent->y());

            return true;
        }
        else if(event->type() == QEvent::MouseButtonPress)
        {
            //frame move by mouse press
            QMouseEvent *overviewPressEvent = static_cast<QMouseEvent*>(event);
            doOverviewMoveEvent(overviewPressEvent->x(), overviewPressEvent->y());

            return true;
        }
        else
        {
            return false;
        }
    }
    if(obj == zoomLabel)
    {
        if(event->type() == QEvent::MouseButtonPress)
        {
            //specification of mark position by mouse press
            QMouseEvent *zoomPressEvent = static_cast<QMouseEvent*>(event);

            placePointFromZoomArea(zoomPressEvent->x(), zoomPressEvent->y());

            return true;
        }
        else
        {
            return false;
        }
    }
    else if(obj == imageLabel)
    {
        if(event->type() == QEvent::MouseMove && memory.getCornersCrd())
        {
            //showing object coordinates by mouse move
            QMouseEvent *moveEvent = static_cast<QMouseEvent*>(event);

            showObjectCoords(moveEvent->x(), moveEvent->y());

            return true;
        }
        if(event->type() == QEvent::MouseButtonPress)
        {
            //inserting marks into image by mouse press
            QMouseEvent *imagePressEvent = static_cast<QMouseEvent*>(event);

            if(markingTool)
            {
                if(markingTool->marking() && imagePressEvent->button() == Qt::LeftButton)
                {
                    if(zoomInt != 1)
                    {
                        getZoomArea(imagePressEvent->x(), imagePressEvent->y());
                    }
                    else
                    {
                        movePoint(imagePressEvent->x() - 11, imagePressEvent->y() - 43);
                    }
                }
            }
            if(memory.getActionName() == "scalebarAction")
            {
                drawScalebar(imagePressEvent->x(), imagePressEvent->y());
            }

            return true;
        }
        else
        {
            return false;
        }
    }
    else
    {
         return QMainWindow::eventFilter(obj, event);
    }
}

//showing object coordinates in status bar
void SIMphoto::showObjectCoords(int x, int y)
{
    QString st;
    double xObj, yObj;

    xObj = cornersCoords(0, 0) + memory.getObjectCoordsKey()(0) * x - memory.getObjectCoordsKey()(1) * y;
    yObj = cornersCoords(0, 1) - memory.getObjectCoordsKey()(0) * y - memory.getObjectCoordsKey()(1) * x;

    st.append(tr("Object Coordinates: ")).append("X = ").append(QString::number(xObj, 'f', 3))
                                         .append(", Y = ").append(QString::number(yObj, 'f', 3));
    statusBar()->showMessage(st,0);
}

//setting scroll bars position by moving the frame
void SIMphoto::doOverviewMoveEvent(int x, int y)
{
    double coeff;
    double cursorPosX, cursorPosY;

    coeff = 1.0 * image->width() / overviewLabel->width();

    cursorPosX = x - overviewLabel->getFrame()->width() / 2.0;
    cursorPosY = y - overviewLabel->getFrame()->height() / 2.0;

    imageScrollArea->horizontalScrollBar()->setSliderPosition((int)(coeff * cursorPosX));
    imageScrollArea->verticalScrollBar()->setSliderPosition((int)(coeff * cursorPosY));
}

void SIMphoto::disableOtherActions()
{
    imageMenu->setEnabled(false);
    editMenu->setEnabled(false);
    photomapMenu->setEnabled(false);
    functionsMenu->setEnabled(false);

    imageToolBar->setEnabled(false);
    photomapToolBar->setEnabled(false);
    functionsToolBar->setEnabled(false);
}

void SIMphoto::enableOtherActions()
{
    imageMenu->setEnabled(true);
    editMenu->setEnabled(true);
    photomapMenu->setEnabled(true);
    functionsMenu->setEnabled(true);

    imageToolBar->setEnabled(true);
    photomapToolBar->setEnabled(true);
    functionsToolBar->setEnabled(true);
}

//Cross cursor is helpful during marking and positioning scale bar
void SIMphoto::setCrossCursor()
{
    QPixmap crossCursorBitmap("images/blackCursor.bmp");

    if(blackCursorColor)
    {
        crossCursor = QCursor(crossCursorBitmap, crossCursorBitmap, -1, -1);
    }
    else
    {
        QPixmap whiteCursorPixmap("images/whiteCursor.bmp");
        whiteCursorPixmap.setMask(crossCursorBitmap);
        crossCursor = QCursor(whiteCursorPixmap, -1, -1);
    }

    if(markingTool)
    {
        if(markingTool->marking())
            imageScrollArea->widget()->setCursor(crossCursor);
    }

    if(memory.getActionName() == "scalebarAction")
        imageScrollArea->widget()->setCursor(crossCursor);
}

void SIMphoto::setCursorColor()
{
    //bool
    blackCursorColor = cursorColorDialog->colorIsBlack();

    delete cursorColorDialog;
    cursorColorDialog = 0;

    setCrossCursor();
}

void SIMphoto::keyPressEvent(QKeyEvent *event)
{
    if(markingTool && !zoomIsActive)
    {
        if(markingTool->marking() && event->key() == Qt::Key_F1)
        {
            if(zoomInt == 3 || zoomInt == 5)
            {
                //Zoom is turned off.
                zoomInt = 1;
                statusBar()->showMessage(tr("Zoom is disabled."), 4000);
            }
            else
            {
                //Zoom is turned on.
                zoomInt = 3;
                statusBar()->showMessage(tr("Zoom is enabled."), 4000);
            }
        }
        if(markingTool->marking() && event->key() == Qt::Key_Q)
        {
            //point mark ++
            markingTool->getSpinBox()->setValue(markingTool->getSpinBox()->value() + 1);

            if(memory.getActionName() == "measureAction")
                measureTool->getSpinBox()->setValue(2);

            if(memory.getActionName() == "similarTransAction")
                similarTransDialog->getPointSpinBox()->setValue(2);
        }
        if(markingTool->marking() && event->key() == Qt::Key_A)
        {
            //point mark --
            markingTool->getSpinBox()->setValue(markingTool->getSpinBox()->value() - 1);

            if(memory.getActionName() == "measureAction")
                measureTool->getSpinBox()->setValue(1);

            if(memory.getActionName() == "similarTransAction")
                similarTransDialog->getPointSpinBox()->setValue(1);
        }
    }
}

//drawing lines into the image (network method)
void SIMphoto::drawNetworkPath()
{
    QImage *drawing = new QImage(*image);
    QPainterPath *path = new QPainterPath;

    //coordinates of network points
    Matrix<int, 4, 2> P;
    P << markingTool->getListOfPoints().at(0)->x(), markingTool->getListOfPoints().at(0)->y(),
         markingTool->getListOfPoints().at(1)->x(), markingTool->getListOfPoints().at(1)->y(),
         markingTool->getListOfPoints().at(2)->x(), markingTool->getListOfPoints().at(2)->y(),
         markingTool->getListOfPoints().at(3)->x(), markingTool->getListOfPoints().at(3)->y();

    //drawing path
    path->moveTo(P(0, 0) + 11, P(0, 1) + 43);

    path->lineTo(P(1, 0) + 11, P(1, 1) + 43);
    path->lineTo(P(2, 0) + 11, P(2, 1) + 43);
    path->lineTo(P(3, 0) + 11, P(3, 1) + 43);
    path->lineTo(P(0, 0) + 11, P(0, 1) + 43);

    if(memory.getDiagonal() == 0)
    {
        path->lineTo(P(2, 0) + 11, P(2, 1) + 43);
        path->moveTo(P(3, 0) + 11, P(3, 1) + 43);
        path->lineTo(P(1, 0) + 11, P(1, 1) + 43);
    }

    if(memory.getDiagonal() == 1)
    {
        path->lineTo(P(2, 0) + 11, P(2, 1) + 43);
    }

    if(memory.getDiagonal() == 2)
    {
        path->moveTo(P(3, 0) + 11, P(3, 1) + 43);
        path->lineTo(P(1, 0) + 11, P(1, 1) + 43);
    }

    QPainter *painter = new QPainter(drawing);
    painter->setPen(QPen(QColor(255, 255, 255), 1, Qt::SolidLine,Qt::FlatCap, Qt::MiterJoin));
    painter->setRenderHint(QPainter::Antialiasing, true);
    painter->drawPath(*path);

    imageLabel->setPixmap(QPixmap::fromImage(*drawing));
    imageLabel->adjustSize();

    painter->end();

    delete drawing;
    delete painter;
    delete path;
}

//drawing lines into the image (for distances)
void SIMphoto::drawDistLines()
{
    QImage *drawing = new QImage(*image);

    QPainter *painter = new QPainter(drawing);
    painter->setPen(QPen(QColor(255, 255, 255), 1, Qt::SolidLine,Qt::FlatCap, Qt::MiterJoin));
    painter->setRenderHint(QPainter::Antialiasing, true);
    painter->drawLine(markingTool->getListOfPoints().at(0)->x() + 11, markingTool->getListOfPoints().at(0)->y() + 43,
                      markingTool->getListOfPoints().at(1)->x() + 11, markingTool->getListOfPoints().at(1)->y() + 43);

    if(memory.getActionName() == "twoDistMethod")
        painter->drawLine(markingTool->getListOfPoints().at(1)->x() + 11, markingTool->getListOfPoints().at(1)->y() + 43,
                          markingTool->getListOfPoints().at(2)->x() + 11, markingTool->getListOfPoints().at(2)->y() + 43);

    imageLabel->setPixmap(QPixmap::fromImage(*drawing));
    imageLabel->adjustSize();

    painter->end();

    delete drawing;
    delete painter;
}

//drawing marks, lines and scale bar into the overview
void SIMphoto::drawOverview()
{
    double ratio;
    ratio = 1.0 * overviewLabel->width() / image->width();

    QImage *drawing = new QImage(image->scaled(400, 400, Qt::KeepAspectRatio, Qt::FastTransformation));
    QPainter *painter = new QPainter(drawing);

    painter->setRenderHint(QPainter::Antialiasing, true);

    //drawing lines
    if(memory.getActionName() == "networkMethod" && markingTool->getPointsCount() == 4)
    {
        painter->setPen(QPen(QColor(255, 255, 255), 1, Qt::SolidLine,Qt::FlatCap, Qt::MiterJoin));
        painter->drawPath(overviewPath(ratio));
    }

    if((memory.getActionName() == "oneDistMethod" && markingTool->getPointsCount() == 2) ||
       (memory.getActionName() == "twoDistMethod" && markingTool->getPointsCount() == 3) ||
       (memory.getActionName() == "measureAction" && markingTool->getPointsCount() == 2) ||
       (memory.getActionName() == "similarTransAction" && markingTool->getPointsCount() == 2))
    {
        painter->setPen(QPen(QColor(255, 255, 255), 1, Qt::SolidLine,Qt::FlatCap, Qt::MiterJoin));
        painter->drawLine((int)((markingTool->getListOfPoints().at(0)->x() + 11) * ratio), (int)((markingTool->getListOfPoints().at(0)->y() + 43) * ratio),
                          (int)((markingTool->getListOfPoints().at(1)->x() + 11) * ratio), (int)((markingTool->getListOfPoints().at(1)->y() + 43) * ratio));
    }

    if(memory.getActionName() == "twoDistMethod" && markingTool->getPointsCount() == 3)
    {
        painter->drawLine((int)((markingTool->getListOfPoints().at(1)->x() + 11) * ratio), (int)((markingTool->getListOfPoints().at(1)->y() + 43) * ratio),
                          (int)((markingTool->getListOfPoints().at(2)->x() + 11) * ratio), (int)((markingTool->getListOfPoints().at(2)->y() + 43) * ratio));
    }

    //drawing marks
    if(markingTool)
    {
        for(int i = 1; i <= markingTool->getPointsCount(); i++)
        {
            QString pixmapPath("o.bmp");
            pixmapPath.prepend(QString::number(i)).prepend("images/overviewPoints/");
            painter->drawPixmap((int)(markingTool->getListOfPoints().at(i - 1)->x() * ratio),
                                (int)(markingTool->getListOfPoints().at(i - 1)->y() * ratio), pixmapPath);
        }
    }

    //drawing scale bar
    if(memory.getActionName() == "scalebarAction")
    {
        QBrush *brush = new QBrush(QColor(255, 255, 255), Qt::SolidPattern);
        painter->setBrush(*brush);
        painter->drawRect((int)(scalebarRect.left() * ratio), (int)(scalebarRect.top() * ratio),
                         (int)(scalebarRect.width() * ratio), (int)(scalebarRect.height() * ratio));

        delete brush;
    }

    overviewLabel->setPixmap(QPixmap::fromImage(*drawing));

    painter->end();

    delete drawing;
    delete painter;
}

//getting network path
QPainterPath SIMphoto::overviewPath(double overviewScale)
{
    QPainterPath path;

    Matrix<int, 4, 2> P;
    P << markingTool->getListOfPoints().at(0)->x(), markingTool->getListOfPoints().at(0)->y(),
         markingTool->getListOfPoints().at(1)->x(), markingTool->getListOfPoints().at(1)->y(),
         markingTool->getListOfPoints().at(2)->x(), markingTool->getListOfPoints().at(2)->y(),
         markingTool->getListOfPoints().at(3)->x(), markingTool->getListOfPoints().at(3)->y();

    path.moveTo((P(0, 0) + 11) * overviewScale, (P(0, 1) + 43) * overviewScale);

    path.lineTo((P(1, 0) + 11) * overviewScale, (P(1, 1) + 43) * overviewScale);
    path.lineTo((P(2, 0) + 11) * overviewScale, (P(2, 1) + 43) * overviewScale);
    path.lineTo((P(3, 0) + 11) * overviewScale, (P(3, 1) + 43) * overviewScale);
    path.lineTo((P(0, 0) + 11) * overviewScale, (P(0, 1) + 43) * overviewScale);

    if(memory.getDiagonal() == 0)
    {
        path.lineTo((P(2, 0) + 11) * overviewScale, (P(2, 1) + 43) * overviewScale);
        path.moveTo((P(3, 0) + 11) * overviewScale, (P(3, 1) + 43) * overviewScale);
        path.lineTo((P(1, 0) + 11) * overviewScale, (P(1, 1) + 43) * overviewScale);
    }

    if(memory.getDiagonal() == 1)
    {
        path.lineTo((P(2, 0) + 11) * overviewScale, (P(2, 1) + 43) * overviewScale);
    }

    if(memory.getDiagonal() == 2)
    {
        path.moveTo((P(3, 0) + 11) * overviewScale, (P(3, 1) + 43) * overviewScale);
        path.lineTo((P(1, 0) + 11) * overviewScale, (P(1, 1) + 43) * overviewScale);
    }

    return path;
}

//creating zoom
void SIMphoto::getZoomArea(int x, int y)
{
    //click position
    int xPosition, yPosition;
    xPosition = x - 20;

    zoomIsActive = true;

    for(int i = 0; i < 41; i++)
    {
        yPosition = y - 20;
        for(int j = 0; j < 41; j++)
        {
            if(xPosition >= 0 && yPosition >= 0 && xPosition < image->width() && yPosition < image->height())
            {
                zoomImage->setPixel(i, j, image->pixel(xPosition, yPosition));
            }
            else
            {
                zoomImage->setPixel(i, j, Qt::black);
            }

            yPosition++;
        }

        xPosition++;
    }

    markingTool->getPoint()->hide();

    zoomLabel->setPixmap(QPixmap::fromImage(zoomImage->scaled(zoomInt * 41, zoomInt * 41, Qt::KeepAspectRatio, Qt::FastTransformation)));
    zoomLabel->setGeometry(x - (zoomInt * 41 - 1) / 2, y - (zoomInt * 41 - 1) / 2, zoomInt * 41, zoomInt * 41);
    zoomLabel->show();
}

//placing mark on click into zoom area
void SIMphoto::placePointFromZoomArea(int x, int y)
{
    int xPosition, yPosition;

    xPosition = zoomLabel->x() + (int)(x / zoomInt) + (zoomInt * 41 - 1) / 2 - 20 - 11;
    yPosition = zoomLabel->y() + (int)(y / zoomInt) + (zoomInt * 41 - 1) / 2 - 20 - 43;

    movePoint(xPosition, yPosition);
    zoomLabel->hide();

    zoomIsActive = false;
}

void SIMphoto::createTableOfPoints()
{
    tableOfPoints = new TableOfPoints(markingTool->getPointsCount(), this);

    if(memory.getActionName() == "networkMethod")
    {
        Matrix<double, 9, 2> coords;
        coords.setZero();

        //getting network coordinates
        for(int i = 0; i < 4; i++)
        {
            coords(i, 0) = memory.getNetworkCoordinates()(i, 0);
            coords(i, 1) = memory.getNetworkCoordinates()(i, 1);
        }

        tableOfPoints->insertData(coords, markingTool->getListOfPoints(), true);
    }
    else
    {
        tableOfPoints->insertData(memory.getObjectCoordinates(), markingTool->getListOfPoints(), false);
    }

    tableOfPoints->show();

    connect(tableOfPoints->getTransformButton(), SIGNAL(clicked()), this, SLOT(checkInputData()));
    connect(tableOfPoints->getSkipButton(), SIGNAL(clicked()), this, SLOT(closeTableOfPoints()));
    connect(tableOfPoints->getSaveTableButton(), SIGNAL(clicked()), this, SLOT(saveTableData()));
    connect(tableOfPoints->getLoadCoordsButton(), SIGNAL(clicked()), this, SLOT(loadCoordinates()));
    connect(tableOfPoints->getShowResidualsButton(), SIGNAL(clicked()), this, SLOT(showResiduals()));
}

void SIMphoto::closeTableOfPoints()
{
    if(memory.getActionName() == "controlPointsMethod")
    {
        Matrix<double, 9, 2> coords;
        coords.setZero();

        for(int i = 0; i < markingTool->getPointsCount(); i++)
        {
            coords(i, 0) = tableOfPoints->getObjectXList().at(i)->value();
            coords(i, 1) = tableOfPoints->getObjectYList().at(i)->value();
        }

        memory.setObjectCoordinates(coords);
    }

    delete tableOfPoints;
}

//before colinear transformation
void SIMphoto::checkInputData()
{
    //getting points that are used
    getCheckedPoints();

    if(checkedPointsCount < 4)
    {
        QMessageBox::information(this, tr("Warning"),
                                      tr("Few points!"));
        return;
    }

    createPixelSizeDialog();
}

void SIMphoto::getCheckedPoints()
{
    checkedPointsCount = 0;
    pointIndex.setZero();

    for(int i = 0; i < markingTool->getPointsCount(); i++)
    {
        if(tableOfPoints->getCheckBoxList().at(i)->isChecked())
        {
            checkedPointsCount++;
            pointIndex(checkedPointsCount - 1) = i;
        }
        else
        {
            tableOfPoints->getTable()->item(i, 6)->setData(Qt::DisplayRole, "");
            tableOfPoints->getTable()->item(i, 7)->setData(Qt::DisplayRole, "");
        }
    }
}

void SIMphoto::createPixelSizeDialog()
{
    pixelSizeDialog = new PixelSizeDialog(this);
    estimateScale();
    pixelSizeDialog->show();

    connect(pixelSizeDialog->getOkButton(), SIGNAL(clicked()), this, SLOT(getSizeOfPixel()));
    connect(pixelSizeDialog, SIGNAL(rejected()), this, SLOT(closePixelSizeDialog()));
}

void SIMphoto::closePixelSizeDialog()
{
    delete pixelSizeDialog;
}

void SIMphoto::estimateScale()
{
    int PX, PY, min = 0, max = 0;
    double dist, minDist, maxDist, distM, distP;

    minDist = 10000.0;
    maxDist = 0.0;

    //getting mark with min and max distance from upper left corner of image
    //Distance between marks will be used for computing pixel size
    for(int i = 0; i < checkedPointsCount; i++)
    {
        PX = markingTool->getListOfPoints().at(i)->x() + 11;
        PY = markingTool->getListOfPoints().at(i)->y() + 43;

        dist = sqrt(PX * PX + PY * PY);

        if(dist < minDist)
        {
            minDist = dist;
            min = i;
        }

        if(dist > maxDist)
        {
            maxDist = dist;
            max = i;
        }
    }

    //getting pixel and meter coordinates of two marks
    Matrix<int, 2, 2> P;
    Matrix<double, 2, 2> M;

    P << markingTool->getListOfPoints().at(min)->x() + 11, markingTool->getListOfPoints().at(min)->y() + 43,
         markingTool->getListOfPoints().at(max)->x() + 11, markingTool->getListOfPoints().at(max)->y() + 43;

    M << tableOfPoints->getObjectXList().at(min)->value(), tableOfPoints->getObjectYList().at(min)->value(),
         tableOfPoints->getObjectXList().at(max)->value(), tableOfPoints->getObjectYList().at(max)->value();

    distP = sqrt((P(1, 0) - P(0, 0)) * (P(1, 0) - P(0, 0)) + (P(1, 1) - P(0, 1)) * (P(1, 1) - P(0, 1)));
    distM = sqrt((M(1, 0) - M(0, 0)) * (M(1, 0) - M(0, 0)) + (M(1, 1) - M(0, 1)) * (M(1, 1) - M(0, 1)));

    double size;

    //pixel size
    size = distM / distP;

    if(size < 1e-4)
    {
        pixelSizeDialog->getPixelSizeBox()->setValue(0.0001);
    }
    else
    {
        pixelSizeDialog->getPixelSizeBox()->setValue(size);
    }
}

void SIMphoto::getSizeOfPixel()
{
    if(pixelSizeDialog->getPixelSizeBox()->value() < 1e-4)
    {
            QMessageBox::information(this, tr("Warning"),
                                      tr("Pixel size cannot be zero!"));
            return;
    }

    memory.setPreviousPixelSize(memory.getPixelSize());
    memory.setPixelSize(pixelSizeDialog->getPixelSizeBox()->value());

    delete pixelSizeDialog;

    doColinearTransformation();
}

void SIMphoto::doColinearTransformation()
{
    //control points coordinates
    xx.resize(checkedPointsCount, 1);
    yy.resize(checkedPointsCount, 1);
    XX.resize(checkedPointsCount, 1);
    YY.resize(checkedPointsCount, 1);

    for(int i = 0; i < checkedPointsCount; i++)
    {
        xx(i) = markingTool->getListOfPoints().at(pointIndex(i))->x() + 11.0;
        yy(i) = markingTool->getListOfPoints().at(pointIndex(i))->y() + 43.0;
        XX(i) = tableOfPoints->getObjectXList().at(pointIndex(i))->value() / memory.getPixelSize();
        YY(i) = - tableOfPoints->getObjectYList().at(pointIndex(i))->value() / memory.getPixelSize();
    }

    //saving current state
    previousCornersCoords = cornersCoords;
    memory.setPreviousImage(image);

    QSize newImageSize;
    newImageSize = newImageSizeForColTrans();

    image->~QImage();
    image = new QImage(newImageSize.width(), newImageSize.height(), QImage::Format_ARGB32);

    memory.setPreviousObjectCoordsKey(memory.getObjectCoordsKey());
    memory.setPreviousCornersCrd(memory.getCornersCrd());

    if(memory.getActionName() == "controlPointsMethod")
    {
        memory.setCornersCrd(true);
        CoeffForObjCrd();
    }
    else
    {
        memory.setCornersCrd(false);
        statusBar()->clearMessage();
    }

    //resampling...
    QRgb black = qRgba(0, 0, 0, 255);
    image->fill(black);

    int indexX, indexY, red, green, blue;
    double alpha, beta, transformX, transformY, Tx, Ty;
    QRgb rgbA, rgbB, rgbC, rgbD, newRgb;

    getTransformCoefficients(XX, YY, xx, yy);

    QProgressDialog progress(tr("Resampling image..."), tr("Abort Resampling"), 0, newImageSize.width(), this);
    progress.setWindowModality(Qt::WindowModal);
    progress.setMinimumDuration(0);

    bool progressAborted = false;

    for(int i = 0; i < newImageSize.width(); i++)
    {
        progress.setValue(i);

        if(progress.wasCanceled())
        {
            imageWasModified();
            undo();

            if(overviewLabel)
                drawOverview();

            if(memory.getActionName() == "networkMethod")
               drawNetworkPath();

            progressAborted = true;

            break;
        }

        for(int j = 0; j < newImageSize.height(); j++)
        {
            Tx = i - translationX;
            Ty = j - translationY;

            transformX = (coeff(0) * Tx / maxCoordinate + coeff(1) * Ty / maxCoordinate + coeff(2)) / (coeff(6) * Tx / maxCoordinate + coeff(7) * Ty / maxCoordinate + 1);
            transformY = (coeff(3) * Tx / maxCoordinate + coeff(4) * Ty / maxCoordinate + coeff(5)) / (coeff(6) * Tx / maxCoordinate + coeff(7) * Ty / maxCoordinate + 1);

            transformX = transformX * maxCoordinate;
            transformY = transformY * maxCoordinate;

            indexX = (int) floor(transformX);
            indexY = (int) floor(transformY);

            if((indexX >= 1 && indexX < memory.getPreviousImage()->width() - 1) && (indexY >= 1 && indexY < memory.getPreviousImage()->height() - 1))
            {
                alpha = transformX - indexX;
                beta = transformY - indexY;

                rgbA = memory.getPreviousImage()->pixel(indexX, indexY);
                rgbB = memory.getPreviousImage()->pixel(indexX + 1, indexY);
                rgbC = memory.getPreviousImage()->pixel(indexX, indexY + 1);
                rgbD = memory.getPreviousImage()->pixel(indexX + 1, indexY + 1);

                red = (int) round((1 - beta) * ((1 - alpha) * qRed(rgbA) +  alpha * qRed(rgbB)) + beta * ((1 - alpha) * qRed(rgbC) + alpha * qRed(rgbD)));
                green = (int) round((1 - beta) * ((1 - alpha) * qGreen(rgbA) +  alpha * qGreen(rgbB)) + beta * ((1 - alpha) * qGreen(rgbC) + alpha * qGreen(rgbD)));
                blue = (int) round((1 - beta) * ((1 - alpha) * qBlue(rgbA) +  alpha * qBlue(rgbB)) + beta * ((1 - alpha) * qBlue(rgbC) + alpha * qBlue(rgbD)));

                newRgb = qRgba(red, green, blue, 255);

                image->setPixel(i, j, newRgb);
            }
        }
    }

    progress.setValue(newImageSize.width());

    if(!progressAborted)
    {
        imageLabel->setPixmap(QPixmap::fromImage(*image));
        imageLabel->adjustSize();

        closeTableOfPoints();
        closeMarkingTool();

        if(overviewLabel)
            overviewLabel->setOverview(image, imageScrollArea->size(), imageScrollArea->widget()->pos());

        imageWasModified();
    }
}

//calculating size of a new image (for colinear transformation)
QSize SIMphoto::newImageSizeForColTrans()
{
    QSize newSize;
    double minX, minY, maxX, maxY;

    Matrix<double, 4, 2> imageCoord, corner;

    //coordinates of corners
    imageCoord << 0, 0,
                  image->width() - 1, 0,
                  image->width() - 1, image->height() - 1,
                  0, image->height() - 1;

    getTransformCoefficients(xx, yy, XX, YY);

    //calculating new corners
    for (int i = 0; i < 4; i++)
    {
        corner(i, 0) = (coeff(0) * imageCoord(i, 0) / maxCoordinate + coeff(1) * imageCoord(i, 1) / maxCoordinate + coeff(2)) / (coeff(6) * imageCoord(i, 0) / maxCoordinate + coeff(7) * imageCoord(i, 1) / maxCoordinate + 1);
        corner(i, 1) = (coeff(3) * imageCoord(i, 0) / maxCoordinate + coeff(4) * imageCoord(i, 1) / maxCoordinate + coeff(5)) / (coeff(6) * imageCoord(i, 0) / maxCoordinate + coeff(7) * imageCoord(i, 1) / maxCoordinate + 1);

        corner(i, 0) = corner(i, 0) * maxCoordinate;
        corner(i, 1) = corner(i, 1) * maxCoordinate;

        if (i == 0)
        {
            minX = corner(i, 0);
            minY = corner(i, 1);
            maxX = minX;
            maxY = minY;
        }
        else
        {
            minX = std::min(minX, corner(i, 0));
            minY = std::min(minY, corner(i, 1));
            maxX = std::max(maxX, corner(i, 0));
            maxY = std::max(maxY, corner(i, 1));
        }

    }

    translationX = - minX;
    translationY = - minY;

    //corners coordinate of the object
    if(memory.getActionName() == "controlPointsMethod")
    {
        cornersCoords(0, 0) = minX * memory.getPixelSize();
        cornersCoords(0, 1) = - minY * memory.getPixelSize();
        cornersCoords(1, 0) = maxX * memory.getPixelSize();
        cornersCoords(1, 1) = - maxY * memory.getPixelSize();
    }

    newSize.setWidth((int) round(maxX - minX + 1));
    newSize.setHeight((int) round(maxY - minY + 1));

    return newSize;
}

//calculating residuals
void SIMphoto::showResiduals()
{
    getCheckedPoints();

    if(checkedPointsCount < 5)
    {
        QMessageBox::information(this, "Warning", "Few points!");
        return;
    }

    //control points coordinates
    xx.resize(checkedPointsCount, 1);
    yy.resize(checkedPointsCount, 1);
    XX.resize(checkedPointsCount, 1);
    YY.resize(checkedPointsCount, 1);

    for(int i = 0; i < checkedPointsCount; i++)
    {
        xx(i) = markingTool->getListOfPoints().at(pointIndex(i))->x() + 11.0;
        yy(i) = markingTool->getListOfPoints().at(pointIndex(i))->y() + 43.0;
        XX(i) = tableOfPoints->getObjectXList().at(pointIndex(i))->value();
        YY(i) = - tableOfPoints->getObjectYList().at(pointIndex(i))->value();
    }

    getTransformCoefficients(xx, yy, XX, YY);

    //adding residuals to the table of points
    for(int i = 0, c = 0; i < markingTool->getPointsCount(); i++)
    {
        if(i == pointIndex(c))
        {
            tableOfPoints->getTable()->item(i, 6)->setData(Qt::DisplayRole, QString::number(v(c * 2) * maxCoordinate, 'f', 4));
            tableOfPoints->getTable()->item(i, 6)->setFont(QFont("Arial", 9, QFont::Bold));
            tableOfPoints->getTable()->item(i, 7)->setData(Qt::DisplayRole, QString::number(- v(c * 2 + 1) * maxCoordinate, 'f', 4));
            tableOfPoints->getTable()->item(i, 7)->setFont(QFont("Arial", 9, QFont::Bold));
            c++;
        }
        else
        {
            tableOfPoints->getTable()->item(i, 6)->setData(Qt::DisplayRole,"");
            tableOfPoints->getTable()->item(i, 7)->setData(Qt::DisplayRole,"");
        }
    }

    tableOfPoints->getTransformButton()->setEnabled(true);
}

//calculating coefficients of colinear transformation
void SIMphoto::getTransformCoefficients(Matrix<double, Eigen::Dynamic, 1> x1, Matrix<double, Eigen::Dynamic, 1> y1,
                                        Matrix<double, Eigen::Dynamic, 1> x2, Matrix<double, Eigen::Dynamic, 1> y2)
{
    Matrix<double, Eigen::Dynamic, Eigen::Dynamic> A;
    Matrix<double, Eigen::Dynamic, 1> b;
    Matrix<double, 4, 1> maximum;

    //searching max coordinate value (coordinates will be reduced)
    maximum(0) = x1.maxCoeff();
    maximum(1) = y1.maxCoeff();
    maximum(2) = x2.maxCoeff();
    maximum(3) = y2.maxCoeff();

    maxCoordinate = maximum.maxCoeff();

    for(int i = 0; i < checkedPointsCount; i++)
    {
        //coordinates are reduced (max coordinate is not zero because of raster coord. system)
        x1(i) = x1(i) / maxCoordinate;
        y1(i) = y1(i) / maxCoordinate;
        x2(i) = x2(i) / maxCoordinate;
        y2(i) = y2(i) / maxCoordinate;
    }

    //vector of transformation coefficients
    coeff.resize(8, 1);

    A.resize(8, 8);
    b.resize(8, 1);

    A.setZero();

    //calculating approximate coefficients
    for(int i = 0, c = 0; i < 8; i += 2, c++)
    {
        A(i, 0) = x1(c);
        A(i, 1) = y1(c);
        A(i, 2) = 1.0;
        A(i, 6) = - x2(c) * x1(c);
        A(i, 7) = - x2(c) * y1(c);

        A(i + 1, 3) = x1(c);
        A(i + 1, 4) = y1(c);
        A(i + 1, 5) = 1.0;
        A(i + 1, 6) = - y2(c) * x1(c);
        A(i + 1, 7) = - y2(c) * y1(c);

        b(i) = x2(c);
        b(i + 1) = y2(c);
    }

    coeff = A.inverse() * b;

    //if points count > 4, adjustment is computed
    if(checkedPointsCount > 4)
    {
        Matrix<double, Eigen::Dynamic, 1> L;
        Matrix<double, Eigen::Dynamic, 1> L0;
        Matrix<double, Eigen::Dynamic, 1> l;
        Matrix<double, 8, 8> N;
        Matrix<double, 8, 8> N_inverse;
        Matrix<double, 8, 1> y;

        Matrix<double, 8, 1> dx;

        L.resize(checkedPointsCount * 2, 1);
        L0.resize(checkedPointsCount * 2, 1);
        l.resize(checkedPointsCount * 2, 1);
        v.resize(checkedPointsCount * 2, 1);
        A.resize(checkedPointsCount * 2, 8);

        double AA, BB, CC;

        //iteration...
        for(int iter = 0; iter < 4; iter++)
        {
            A.setZero();

            for(int i = 0; i < checkedPointsCount; i++)
            {
                AA = x1(i) * coeff(0) + y1(i) * coeff(1) + coeff(2);
                BB = x1(i) * coeff(3) + y1(i) * coeff(4) + coeff(5);
                CC = x1(i) * coeff(6) + y1(i) * coeff(7)+ 1;

                A(i * 2, 0) = x1(i) / CC;
                A(i * 2, 1) = y1(i) / CC;
                A(i * 2, 2) = 1 / CC;
                A(i * 2, 6) = - AA * x1(i) / (CC * CC);
                A(i * 2, 7) = - AA * y1(i) / (CC * CC);

                A(i * 2 + 1, 3) = x1(i) / CC;
                A(i * 2 + 1, 4) = y1(i) / CC;
                A(i * 2 + 1, 5) = 1 / CC;
                A(i * 2 + 1, 6) = - BB * x1(i) / (CC * CC);
                A(i * 2 + 1, 7) = - BB * x1(i) / (CC * CC);

                L(i * 2) = x2(i);
                L(i * 2 + 1) = y2(i);

                L0(i * 2) = AA / CC;
                L0(i * 2 + 1) = BB / CC;
            }

            l = L - L0;
            N = A.transpose() * A;
            N_inverse = N.inverse();
            y = A.transpose() * l;

            dx = N_inverse * y;
            coeff = coeff + dx;

            v = A * dx - l;
        }
    }
}

void SIMphoto::imageWasModified()
{
    memory.addNumberOfChanges();
    undoAct->setEnabled(true);

    if(memory.getPixelSize() >= 1e-4)
    {
        measureAct->setEnabled(true);
        scalebarAct->setEnabled(true);
        printAct->setEnabled(true);
    }
}

void SIMphoto::saveTableData()
{
    QString path;
    getInitPath(path);

    QString fileName = QFileDialog::getSaveFileName(this, tr("Save Table"),
                                     path,
                                     "TXT (*.txt *.TXT)");

    QFile file(fileName);

    if(!fileName.isEmpty())
    {
        if(file.open(QFile::WriteOnly | QFile::Truncate))
        {
            QTextStream output(&file);
            QString useStr;

            output << tr("Use") << "\t" << tr("Point") << "\t" << "X[m]" << "\t" << "Y[m]"
                   << "\t" << "x[p]" << "\t" << "y[p]" << "\t" << tr("Residual X[m]") << "\t" << tr("Residual Y[m]") << "\r\n";

            for(int i = 0; i < markingTool->getPointsCount(); i++)
            {
                if(tableOfPoints->getCheckBoxList().at(i)->isChecked())
                {
                    useStr = tr("YES");
                }
                else
                {
                    useStr = tr("NO");
                }

                output << useStr << "\t"
                       << i + 1 << "\t"
                       << tableOfPoints->getObjectXList().at(i)->value() << "\t"
                       << tableOfPoints->getObjectYList().at(i)->value() << "\t"
                       << markingTool->getListOfPoints().at(i)->x() + 11 << "\t"
                       << markingTool->getListOfPoints().at(i)->y() + 43 << "\t"
                       << tableOfPoints->getTable()->item(i, 6)->text() << "\t" << "\t"
                       << tableOfPoints->getTable()->item(i, 7)->text() << "\r\n";
            }
        }
    }
}

void SIMphoto::loadCoordinates()
{
    QString path;
    getInitPath(path);

    QString fileName = QFileDialog::getOpenFileName(this,
                                     tr("Open File"), path,
                                     "TXT (*.txt *.TXT)");

    QString line;
    QFile file(fileName);

    //specification of separator
    QRegExp sep("\\s+");
    int i = -1;

    Matrix<double, 9, 2> focusedPointXY;
    focusedPointXY.setZero();

    if(file.open(QIODevice::ReadOnly))
    {
        QTextStream in(&file);
        while(!in.atEnd())
        {
            line = in.readLine();

            if(i == -1)
            {
                QString headerStr(tr("pointxy"));
                QString testStr;

                //reading header
                testStr.append(line.section(sep, 0, 0)).append(line.section(sep, 1, 1)).append(line.section(sep, 2, 2));

                if(testStr != headerStr)
                {
                    QMessageBox::information(this, tr("Warning"), tr("Invalid input file! Make sure the file contains the correct header."));
                    return;
                }
            }
            else
            {
                //reading coordinates
                focusedPointXY(i, 0) = line.section(sep, 1, 1).toDouble();
                focusedPointXY(i, 1) = line.section(sep, 2, 2).toDouble();
            }

            i++;

            if(i == markingTool->getPointsCount())
                break;
        }

        if(i < markingTool->getPointsCount())
        {
            QMessageBox::information(this, tr("Warning"), tr("Invalid input file: Few poins!"));
            return;
        }

        closeTableOfPoints();
        markDock->setHidden(true);

        memory.setObjectCoordinates(focusedPointXY);
        focusedPoint = 0;

        setFocusOnPoint();
    }
}

void SIMphoto::setFocusOnPoint()
{
    if(focusedPoint + 1 <= markingTool->getPointsCount())
    {
        markingTool->getSpinBox()->setValue(focusedPoint + 1);

        if(focusedPoint == 0)
        {
            if(overviewLabel)
            {
                delete overviewLabel;
                delete overviewDock;

                overviewLabel = 0;
                overviewDock = 0;
            }
            createOverview();

            overviewLabel->setOverview(image, imageScrollArea->size(), imageScrollArea->widget()->pos());
            drawOverview();
            createCheckPointDialog();

            QMessageBox::information(this, tr("Information"), tr("Now you check whether the coordinates correspond to each mark."));
        }

        imageScrollArea->horizontalScrollBar()->setSliderPosition((int)(markingTool->getListOfPoints().at(focusedPoint)->x() + 11.0 - imageScrollArea->width() / 2.0));
        imageScrollArea->verticalScrollBar()->setSliderPosition((int)(markingTool->getListOfPoints().at(focusedPoint)->y() + 43.0 - imageScrollArea->height() / 2.0));

        checkPointPosition();
    }
    else
    {
        closeCheckPointDialog();
    }
}

void SIMphoto::createCheckPointDialog()
{
    checkPointDialog = new CheckPointDialog;

    checkPointDock = new QDockWidget(tr("Check the Point Position"), this);
    checkPointDock->setAllowedAreas(Qt::LeftDockWidgetArea | Qt::RightDockWidgetArea);
    checkPointDock->setFeatures(QDockWidget::DockWidgetMovable | QDockWidget::DockWidgetFloatable);
    checkPointDock->setWidget(checkPointDialog);

    addDockWidget(Qt::LeftDockWidgetArea, checkPointDock);

    connect(checkPointDialog->getNextPointButton(), SIGNAL(clicked()), this, SLOT(setFocusOnPoint()));
}

void SIMphoto::checkPointPosition()
{
    QString coordsString(tr("Point "));
    coordsString.append(QString::number(focusedPoint + 1)).append(": ")
              .append("X [m] = ").append(QString::number(memory.getObjectCoordinates()(focusedPoint, 0), 'f', 3))
              .append(", Y [m] = ").append(QString::number(memory.getObjectCoordinates()(focusedPoint, 1), 'f', 3));

    checkPointDialog->getFocusedPointCoordLabel()->setText(coordsString);

    focusedPoint++;

    if(focusedPoint + 1 > markingTool->getPointsCount())
        checkPointDialog->getNextPointButton()->setText(tr("Open Point Table"));
}

void SIMphoto::closeCheckPointDialog()
{
    removeDockWidget(markDock);
    addDockWidget(Qt::LeftDockWidgetArea, markDock);

    delete checkPointDialog;
    delete checkPointDock;

    markDock->show();
    createTableOfPoints();
}

//calculating transformation coefficients for getting actual object coordinates
void SIMphoto::CoeffForObjCrd()
{
    Matrix<double, 2, 1> XAB;
    Matrix<double, 2, 2> xab;

    XAB << cornersCoords(1, 0) - cornersCoords(0, 0),
           cornersCoords(0, 1) - cornersCoords(1, 1);

    xab << image->width() - 1, - (image->height() - 1),
           image->height() - 1, image->width() - 1;

    memory.setObjectCoordsKey(xab.inverse() * XAB);
}

void SIMphoto::doAboutAction()
{
    aboutSIMphotoDialog = new AboutDialog(this);
    aboutSIMphotoDialog->show();

    connect(aboutSIMphotoDialog, SIGNAL(rejected()), this, SLOT(closeAboutSIMphotoDialog()));
}

void SIMphoto::closeAboutSIMphotoDialog()
{
    delete aboutSIMphotoDialog;
}

SIMphoto::~SIMphoto()
{

}
